// Author: Ryan Schmidt  (rms@unknownroad.com)  http://www.unknownroad.com
// Copyright (c) 2006. All Rights Reserved.
// The SketchFramework Library is supplied "AS IS". The Author disclaims all warranties, expressed or implied, including, 
// without limitation, the warranties of merchantability and of fitness for any purpose. The Author assume no liability for 
// direct, indirect, incidental, special, exemplary, or consequential damages, which may result from the use of the 
// SketchFramework Library, even if advised of the possibility of such damage. Permission is hereby granted to use, copy, 
// modify, and distribute this source code, or portions hereof, for any purpose, without fee, subject to the following restrictions:
// 1) The origin of this source code must not be misrepresented.
// 2) This Copyright notice may not be removed or altered from any source or altered source distribution.
// The Author specifically permits, without fee, the use of this source code as a component in commercial products.

#ifndef _RMS_BASIC_HANDLE_TARGET_H
#define _RMS_BASIC_HANDLE_TARGET_H

#include <Tool.h>
#include <GLUtil.h>

class BasicHandleTarget : public rmsui::ITool3DTarget
{
public:
	BasicHandleTarget( const Wml::Vector3f & ptPosition = Wml::Vector3f::ZERO)
	{
		m_frame.Origin() = ptPosition;
	}

	virtual ~BasicHandleTarget()
	{
	}

	void Render()
	{
		glPushMatrix();

		Wml::Vector3f & ptOrigin = m_frame.Origin();
		glTranslatef( ptOrigin.X(), ptOrigin.Y(), ptOrigin.Z() );

		Wml::Vector3f & vScale = m_frame.Scale();
		glScalef( vScale.X(), vScale.Y(), vScale.Z() );

		float glMatrix[16];
		rmsui::GLUtil::WmlMatrix3ToGLMatrix(m_frame.FrameInverse(), glMatrix);
		glMultMatrixf(glMatrix);

		GLUquadric * quadric = gluNewQuadric();
		gluCylinder(quadric, 1.0, 0.5, 1.0, 20, 20);
		gluDeleteQuadric( quadric );

		glPopMatrix();
	}

	virtual Wml::Frame GetFrame()
		{ return m_frame; }

	void Translate( const Wml::Vector3f & vTranslate, bool bRelative = true )
		{ m_frame.Translate( vTranslate, bRelative ); }

	void Rotate( const Wml::Matrix3f & rotation )
		{ m_frame.Rotate(rotation); }

protected:
	Wml::Frame m_frame;

};




#endif // _RMS_BASIC_HANDLE_TARGET_H