// Ryan Schmidt - rms@unknownroad.com
// Copyright (c) 2005-2010
// Distributed under the Boost Software License, Version 1.0.
// http://www.boost.org/LICENSE_1_0.txt


#include "stdafx.h"
#include "HUDButtonListeners.h"

#include "rmsdebug.h"

HUDZoomButtonListener::HUDZoomButtonListener( CViewController * pController )
	: m_pViewController(pController)
{
	ASSERT(m_pViewController);
}

void HUDZoomButtonListener::OnDragDelta( const Wml::Vector2f & vDelta )
{
	if (m_pViewController)
		m_pViewController->DollyZoom( vDelta.X() * 10.0f, vDelta.Y() * 10.0f );
}


HUDPanButtonListener::HUDPanButtonListener( CViewController * pController )
	: m_pViewController(pController)
{
	ASSERT(m_pViewController);
}

void HUDPanButtonListener::OnDragDelta( const Wml::Vector2f & vDelta )
{
	if (m_pViewController)
		m_pViewController->Pan( vDelta.X() * 10.0f, -vDelta.Y() * 10.0f );
}



HUDOrbitButtonListener::HUDOrbitButtonListener( CViewController * pController )
	: m_pViewController(pController)
{
	ASSERT(m_pViewController);
}

void HUDOrbitButtonListener::OnDragDelta( const Wml::Vector2f & vDelta )
{
	if (m_pViewController)
		m_pViewController->Orbit( vDelta.X() * 10.0f, vDelta.Y() * 10.0f );
}






HUDStoredViewButtonListener::HUDStoredViewButtonListener( CViewController * pViewController, ViewDirection eViewDirection )
	: m_pViewController(pViewController), 
	  m_eViewDirection( eViewDirection )
{
	ASSERT(m_pViewController);
}

void HUDStoredViewButtonListener::OnCross()
{
	if (!m_pViewController)
		return;
	switch (m_eViewDirection) {
		case Front:
			m_pViewController->ViewFront();
			break;
		case Back:
			m_pViewController->ViewBack();
			break;
		case Top:
			m_pViewController->ViewTop();
			break;
		case Bottom:
			m_pViewController->ViewBottom();
			break;
		case Left:
			m_pViewController->ViewLeft();
			break;
		case Right:
			m_pViewController->ViewRight();
			break;


	}
}


HUDCrossingButtonListener::HUDCrossingButtonListener()
{
}

void HUDCrossingButtonListener::OnCross()
{
	_RMSInfo("Cross!\n");
}




HUDViewToolbarListener::HUDViewToolbarListener( CViewController * pController )
	: m_pViewController(pController)
{
	ASSERT(m_pViewController);
}

void HUDViewToolbarListener::OnHUDToolbarMove( )
{
	if (m_pViewController)
		m_pViewController->ForceRepaint();
}

void HUDViewToolbarListener::OnHUDToolbarHide( )
{
	if (m_pViewController)
		m_pViewController->ForceRepaint();
}