// Ryan Schmidt - rms@unknownroad.com
// Copyright (c) 2005-2010
// Distributed under the Boost Software License, Version 1.0.
// http://www.boost.org/LICENSE_1_0.txt


// MainFrm.cpp : implementation of the CMainFrame class
//

#include "stdafx.h"
#include "BlobShop.h"

#include "MainFrm.h"
#include ".\mainfrm.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif


// CMainFrame

IMPLEMENT_DYNAMIC(CMainFrame, CMDIFrameWnd)

BEGIN_MESSAGE_MAP(CMainFrame, CMDIFrameWnd)
	ON_WM_CREATE()
	ON_WM_DESTROY()
	ON_COMMAND(ID_FILE_NEW, OnFileNew)
END_MESSAGE_MAP()

static UINT indicators[] =
{
	ID_SEPARATOR,           // status line indicator
	ID_INDICATOR_CAPS,
	ID_INDICATOR_NUM,
	ID_INDICATOR_SCRL,
};


// CMainFrame construction/destruction

CMainFrame::CMainFrame()
{
	// TODO: add member initialization code here
}

CMainFrame::~CMainFrame()
{
}


int CMainFrame::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CMDIFrameWnd::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	if (!m_wndToolBar.CreateEx(this, TBSTYLE_FLAT, WS_CHILD | WS_VISIBLE | CBRS_TOP
		| CBRS_GRIPPER | CBRS_TOOLTIPS | CBRS_FLYBY | CBRS_SIZE_DYNAMIC) ||
		!m_wndToolBar.LoadToolBar(IDR_MAINFRAME))
	{
		TRACE0("Failed to create toolbar\n");
		return -1;      // fail to create
	}

	if (!m_AnimationToolBar.CreateEx(this, TBSTYLE_FLAT, WS_CHILD | WS_VISIBLE | CBRS_TOP
		| CBRS_GRIPPER | CBRS_TOOLTIPS | CBRS_FLYBY | CBRS_SIZE_DYNAMIC) ||
		!m_AnimationToolBar.LoadToolBar(IDR_ANIMATION))
	{
		TRACE0("Failed to create animation toolbar\n");
		return -1;      // fail to create
	}

	if (!m_TextureToolBar.CreateEx(this, TBSTYLE_FLAT, WS_CHILD | WS_VISIBLE | CBRS_TOP
		| CBRS_GRIPPER | CBRS_TOOLTIPS | CBRS_FLYBY | CBRS_SIZE_DYNAMIC) ||
		!m_TextureToolBar.LoadToolBar(IDR_TEXTURE))
	{
		TRACE0("Failed to create texture toolbar\n");
		return -1;      // fail to create
	}


	if (!m_wndStatusBar.Create(this) ||
		!m_wndStatusBar.SetIndicators(indicators,
		  sizeof(indicators)/sizeof(UINT)))
	{
		TRACE0("Failed to create status bar\n");
		return -1;      // fail to create
	}

	m_pParamsDialogBar = CParametersDialogBar::Get();
	if (!m_pParamsDialogBar->Create(this, IDD_PARAMETERS_DIALOGBAR,
		 CBRS_TOP | CBRS_GRIPPER | CBRS_TOOLTIPS | CBRS_FLYBY,
		 IDD_PARAMETERS_DIALOGBAR) )
	{
		TRACE0("Failed to create parameters dialog bar\n");
		return -1;      // fail to create
	}

	m_pBlobTreeDialogBar = CBlobTreeEditorDialogBar::Get();
	if (!m_pBlobTreeDialogBar->Create(this, IDD_BLOBTREEEDITOR_DIALOGBAR,
		 CBRS_TOP | CBRS_GRIPPER | CBRS_TOOLTIPS | CBRS_FLYBY,
		 IDD_BLOBTREEEDITOR_DIALOGBAR) )
	{
		TRACE0("Failed to create parameters dialog bar\n");
		return -1;      // fail to create
	}

	// TODO: Delete these three lines if you don't want the toolbar to be dockable
	m_wndToolBar.EnableDocking(CBRS_ALIGN_ANY);
	m_AnimationToolBar.EnableDocking(CBRS_ALIGN_ANY);
	m_TextureToolBar.EnableDocking(CBRS_ALIGN_ANY);
	m_pParamsDialogBar->EnableDocking(CBRS_ALIGN_RIGHT | CBRS_ALIGN_LEFT);
	m_pBlobTreeDialogBar->EnableDocking(CBRS_ALIGN_RIGHT | CBRS_ALIGN_LEFT);
	EnableDocking(CBRS_ALIGN_ANY );
	DockControlBar(&m_wndToolBar);

	// force placement at top
	CRect rect;
	GetClientRect(rect);
	RECT paramsDB = {rect.right, rect.top, rect.right + 1, rect.top + 1 };
	DockControlBar(m_pParamsDialogBar, AFX_IDW_DOCKBAR_RIGHT, &paramsDB );

	// recalc layout will let us dock a bar underneath the above one
	RecalcLayout(TRUE);

	// dock other bar underneath (!)
	GetClientRect(rect);
	RECT blobDB = {rect.right, rect.bottom-2, rect.right + 1, rect.bottom-1 };
	DockControlBar(m_pBlobTreeDialogBar, AFX_IDW_DOCKBAR_RIGHT, &blobDB );

	// ok, now dock texture toolbar to right of default toolbar
	CRect wndRect;
	m_wndToolBar.GetWindowRect(wndRect);
	CRect texRect(wndRect.right + 10, wndRect.top, wndRect.right + 11, wndRect.top + 1 );
	this->DockControlBar(&m_TextureToolBar, (UINT)0, texRect );


	// dock animation toolbar to right of texture toolbar
	m_TextureToolBar.GetWindowRect(wndRect);
	CRect animRect(wndRect.right + 10, wndRect.top, wndRect.right + 11, wndRect.top + 1 );
	this->DockControlBar(&m_AnimationToolBar, (UINT)0, animRect );


	return 0;
}

BOOL CMainFrame::PreCreateWindow(CREATESTRUCT& cs)
{
	if( !CMDIFrameWnd::PreCreateWindow(cs) )
		return FALSE;
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return TRUE;
}


// CMainFrame diagnostics

#ifdef _DEBUG
void CMainFrame::AssertValid() const
{
	CMDIFrameWnd::AssertValid();
}

void CMainFrame::Dump(CDumpContext& dc) const
{
	CMDIFrameWnd::Dump(dc);
}

#endif //_DEBUG


// CMainFrame message handlers


void CMainFrame::SetStatusBarText( CString & text )
{
	m_wndStatusBar.SetWindowText(text);
}


void CMainFrame::OnDestroy()
{
	CMDIFrameWnd::OnDestroy();

	// SaveBarState()
}

void CMainFrame::OnFileNew()
{
	// do nothing!
}

