// Ryan Schmidt - rms@unknownroad.com
// Copyright (c) 2005-2010
// Distributed under the Boost Software License, Version 1.0.
// http://www.boost.org/LICENSE_1_0.txt


#ifndef __RMS_SCENE_FILE_READER_H__
#define __RMS_SCENE_FILE_READER_H__

#include "rmsblobtree.h"

#include <BlobTreeSceneObject.h>
#include <WmlSerializer.h>

extern const Wml::SerializerRecord::RecordID BLOBTREE_LIST_RECORD_ID;

/*
 * maintains the list of BlobTree objects (rootnodes) in the scene. Locate using BLOBTREE_LIST_RECORD_ID
 */
class BlobTreeListRecord
{
public:
	BlobTreeListRecord( ) {};
	~BlobTreeListRecord() {};
	
	void AddBlobTreeRecord( Wml::SerializerRecord * pRootNodeRecord );

	const std::vector< Wml::SerializerRecord::RecordID > & GetRootNodeIDs() { return m_vRootNodeIDs; }

	/*
	 * Serialization
	 */ 
	virtual Wml::SerializerRecord * Serialize( Wml::Serializer * pSerializer );
	virtual bool Serialize( Wml::SerializerRecord * pRecord, Wml::Serializer * pSerializer );

protected:
	std::vector< Wml::SerializerRecord::RecordID > m_vRootNodeIDs;
};


/*
 * There is only one texture manager...
 */
class TextureManagerRecord
{
public:
	TextureManagerRecord() {};
	~TextureManagerRecord() {};

	void SetTextureManagerRecord( Wml::SerializerRecord * pRecord )
		{ m_vTextureManagerID = pRecord->GetRecordID(); }
	
	Wml::SerializerRecord::RecordID TexManRecordID() { return m_vTextureManagerID; } 

	/*
	 * Serialization
	 */ 
	virtual Wml::SerializerRecord * Serialize( Wml::Serializer * pSerializer );
	virtual bool Serialize( Wml::SerializerRecord * pRecord, Wml::Serializer * pSerializer );

protected:
	Wml::SerializerRecord::RecordID m_vTextureManagerID;
};



/*
 * Scense options (there is only one)
 */
class SceneOptionsRecord
{
public:
	SceneOptionsRecord() {};
	~SceneOptionsRecord() {};

	void SetViewRecord( Wml::SerializerRecord * pRecord )
		{ m_vViewRecordID = pRecord->GetRecordID(); }
	
	Wml::SerializerRecord::RecordID ViewRecordID() { return m_vViewRecordID; } 

	/*
	 * Serialization
	 */ 
	virtual Wml::SerializerRecord * Serialize( Wml::Serializer * pSerializer );
	virtual bool Serialize( Wml::SerializerRecord * pRecord, Wml::Serializer * pSerializer );

protected:
	Wml::SerializerRecord::RecordID m_vViewRecordID;
};

// predecls
class CSceneManager;
class CTextureManager;
class CViewController;


class SceneFileReader
{
public:
	SceneFileReader( CSceneManager * pSceneManager, CTextureManager * pTextureManager, CViewController * pViewController );
	~SceneFileReader();

	bool RestoreScene( Wml::Serializer * pSerializer );

protected:
	Wml::Serializer * m_pSerializer;

	CSceneManager * m_pSceneManager;
	CTextureManager * m_pTextureManager;
	CViewController * m_pViewController;


	rmsui_bt::BlobTreeNodeSceneObject * RestoreBTNodeRecord( Wml::SerializerRecord * pRecord, rmsui_bt::BlobTreeNodeSceneObject * pParent );
	void RestoreBTNodeChildren( Wml::SerializerRecord * pRecord, rmsui_bt::BlobTreeNodeSceneObject * pParentNode );

};






#endif // __RMS_SCENE_FILE_READER_H__