// Ryan Schmidt - rms@unknownroad.com
// Copyright (c) 2005-2010
// Distributed under the Boost Software License, Version 1.0.
// http://www.boost.org/LICENSE_1_0.txt


// Standard3DView.cpp : implementation of the CStandard3DView class
//

#include "stdafx.h"
#include "BlobShop.h"

#include "BlobShopDoc.h"
#include "MainFrm.h"
#include "Standard3DView.h"
#include ".\standard3dview.h"

// temp
#include <WmlIntrLin3Sph3.h>
#include <WmlIntrLin3Con3.h>
#include "BasicHandleTarget.h"
#include <PointSceneObject.h>
#include <CappedLinearSweepSceneObject.h>
#include <RevolutionSceneObject.h>
#include <DiscreteScalarField2D.h>
#include <cv.h>
#include <highgui.h>
#include "Image.h"

// tools
#include "Tool3DTranslation.h"
#include "Tool3DRotation.h"
#include "Tool3DSurfaceTranslation.h"
#include "SurfacePatchTransformTool.h"
#include "SurfacePatchDeformTool.h"
#include "SurfaceVectorEditingTool.h"

#include "rmsprofile.h"

#include <FieldCacheManager.h>
#include "SystemConstants.h"

#include "ParametersDialogBar.h"
#include "BlobTreeEditorDialogBar.h"
#include "CachingControlDialog.h"

#include "MeshRefinementOptionsDialog.h"

#include "SceneFileReader.h"

#include "ParameterEditor.h"

#include "glext/glprocs.h"

using namespace rmsui_bt;


#ifdef _DEBUG
#define new DEBUG_NEW
#endif


// CStandard3DView

IMPLEMENT_DYNCREATE(CStandard3DView, CGLView)

BEGIN_MESSAGE_MAP(CStandard3DView, CGLView)
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CGLView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CGLView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CGLView::OnFilePrintPreview)
	ON_WM_CHAR()
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_LBUTTONDBLCLK()
	ON_WM_MBUTTONDOWN()
	ON_WM_MBUTTONUP()
	ON_WM_MBUTTONDBLCLK()
	ON_WM_RBUTTONDOWN()
	ON_WM_RBUTTONUP()
	ON_WM_RBUTTONDBLCLK()
	ON_WM_MOUSEMOVE()
	ON_WM_KEYDOWN()
	ON_WM_KEYUP()
	ON_WM_SYSKEYDOWN()
	ON_WM_SYSKEYUP()
	ON_WM_KILLFOCUS()
	ON_COMMAND(ID_VIEW_FIELDIMAGE, OnViewFieldimage)
	ON_UPDATE_COMMAND_UI(ID_VIEW_FIELDIMAGE, OnUpdateViewFieldimage)
	ON_WM_TIMER()
	ON_WM_SETFOCUS()
	ON_COMMAND(ID_VIEW_ENABLESKETCHING, OnViewEnablesketching)
	ON_UPDATE_COMMAND_UI(ID_VIEW_ENABLESKETCHING, OnUpdateViewEnablesketching)
	ON_COMMAND(ID_VIEW_CACHINGCONTROL, OnViewCachingcontrol)
	ON_COMMAND(ID_STATISTICS_FIELDVALUEAPPROXIMATIONERROR, OnStatisticsFieldvalueapproximationerror)
	ON_COMMAND(ID_STATISTICS_CACHEMEMORYUSAGE, OnStatisticsCachememoryusage)
	ON_COMMAND(ID_STATISTICS_CACHEEVALUATION, OnStatisticsCacheevaluation)
	ON_COMMAND(ID_FILE_SAVE, OnFileSave)
	ON_COMMAND(ID_FILE_SAVE_AS, OnFileSave)
	ON_COMMAND(ID_EDIT_REFINEMESH, OnEditRefinemesh)
	ON_COMMAND(ID_EDIT_INCREASE_MESH_RES, OnEditIncreaseMeshRes)
	ON_COMMAND(ID_EDIT_DECREASE_MESH_RES, OnEditDecreaseMeshRes)
	ON_COMMAND(ID_EDIT_CYCLETOOL, OnEditCycletool)
	ON_COMMAND(ID_VIEW_PERSPECTIVECAMERA, OnViewPerspectivecamera)
	ON_UPDATE_COMMAND_UI(ID_VIEW_PERSPECTIVECAMERA, OnUpdateViewPerspectivecamera)
	ON_COMMAND(ID_EDIT_CLEARCUTPLANE, OnEditClearcutplane)
	ON_COMMAND(ID_EDIT_TOGGLECACHING, OnEditTogglecaching)
	ON_UPDATE_COMMAND_UI(ID_EDIT_TOGGLECACHING, OnUpdateEditTogglecaching)
	ON_COMMAND(ID_VIEWDIRECTIONS_LEFT, OnViewdirectionsLeft)
	ON_COMMAND(ID_VIEWDIRECTIONS_RIGHT, OnViewdirectionsRight)
	ON_COMMAND(ID_VIEWDIRECTIONS_TOP, OnViewdirectionsTop)
	ON_COMMAND(ID_VIEWDIRECTIONS_BOTTOM, OnViewdirectionsBottom)
	ON_COMMAND(ID_VIEWDIRECTIONS_FRONT, OnViewdirectionsFront)
	ON_COMMAND(ID_VIEWDIRECTIONS_BACK, OnViewdirectionsBack)
	ON_COMMAND(ID_WINDOW_RAYTRACE, OnWindowRaytrace)
	ON_COMMAND(ID_EXPORTMESH_OBJFORMAT, OnExportmeshObjformat)
	ON_COMMAND(ID_FILE_OPEN, OnFileOpen)
	ON_COMMAND(ID_EXPORTMESH_MESHLITEFORMAT, OnExportmeshMeshliteformat)
	ON_COMMAND(ID_EDIT_QUICKREFINE, OnEditQuickrefine)
	ON_COMMAND(ID_FILE_EXPORTSELECTION, OnFileExportselection)
	ON_UPDATE_COMMAND_UI(ID_FILE_EXPORTSELECTION, OnUpdateFileExportselection)
	ON_COMMAND(ID_ANIMATION_PLAY, OnAnimationPlay)
	ON_COMMAND(ID_VIEW_SHOWHUD, OnViewShowhud)
	ON_UPDATE_COMMAND_UI(ID_VIEW_SHOWHUD, OnUpdateViewShowhud)
	ON_COMMAND(ID_ANIMATION_STOP, OnAnimationStop)
	ON_COMMAND(ID_ANIMATION_REWIND, OnAnimationRewind)
	ON_COMMAND(ID_ANIMATION_SAVEFRAMES, OnAnimationSaveframes)
	ON_UPDATE_COMMAND_UI(ID_ANIMATION_SAVEFRAMES, OnUpdateAnimationSaveframes)
	ON_COMMAND(ID_ANIMATION_PLAYONEFRAME, OnAnimationPlayoneframe)
	ON_COMMAND(ID_VIEWDIRECTIONS_THREEQUARTER, OnViewdirectionsThreequarter)
//	ON_WM_ACTIVATE()
//ON_WM_INITMENU()
END_MESSAGE_MAP()

// CStandard3DView construction/destruction

CStandard3DView::CStandard3DView()
	: m_pCamera(NULL), m_pViewController(NULL)
{
	// grab control of input even stream (too late for sketch manager!)
	rmsui::CInputEventStream::SetCurrentEventStream(this);

	// initialize scene manager
	CSceneManager::SetCurrentSceneManager(this);
	m_pSceneManager = CSceneManager::Get();

	// [RMS] sketch manager does not handle multiple windows like scene manager does...
	m_pSketchManager = CSketchManager::Get();

	m_pSurfaceManager = new CSurfaceManager();
	m_pRenderManager = new CRenderingManager();
	m_pRenderManager->SetSurfaceManager( m_pSurfaceManager );
	m_pRenderManager->GetTextureManager()->SetSurfaceManager( m_pSurfaceManager );
	m_pHUDManager = new CHUDManager();
	m_pAnimationManager = new CAnimationManager();
	ParameterEditor::Get()->SetRenderManager( m_pRenderManager );

	m_bLeftMouseDown = m_bMiddleMouseDown = m_bRightMouseDown = false;
	m_bAltKeyDown = false;

	m_pSketchManager->SetSketchingEnabled(false);

	m_pToolManager = new rmsui::Tool3DManager();

	// create and register tools
	m_pToolManager->RegisterTool( new rmsui::LocalTranslationTool3D() );
	m_pToolManager->RegisterTool( new rmsui::LocalRotationTool3D() );
	//m_pToolManager->RegisterTool( new rmsui::SurfaceTranslationTool3D() );
	m_pToolManager->RegisterTool( new rmsui_tex::SurfacePatchTransformTool() );
	m_pToolManager->RegisterTool( new rmsui_tex::SurfacePatchDeformTool() );
	m_pToolManager->RegisterTool( new rmsui_tex::SurfaceVectorEditingTool() );

	m_pToolManager->SelectTool( "Translation3D" );

	m_pGestureEngine = new rmsui::GestureEngine();

	m_bAnimating = false;
	m_bStepOneFrameOnly = false;
	m_bDrawHUD = true;
	m_bCaptureFrames = false;

	// initialize field image window stuff
	m_pChildWindows = NULL;

	m_bMenusUpdated = false;
}

CStandard3DView::~CStandard3DView()
{
	if ( m_pViewController )
		delete m_pViewController;
	if (m_pCamera)
		delete m_pCamera;

	if (m_pChildWindows)
		delete m_pChildWindows;
}


// CStandard3DView drawing



void CStandard3DView::ConfigureDevelBuildFeatures()
{
	if (m_bMenusUpdated)
		return;

	if ( CSystemConstants::IsDevelBuild() ) {
		m_bMenusUpdated = true;
		return;
	}

	CMenu * pMenu = GetParentFrame()->GetParentFrame()->GetMenu();
	int nCount = pMenu->GetMenuItemCount();
	if ( nCount > 0) {

		// find shapes menu
		CMenu * pShapesMenu = NULL;
		for ( int i = 0; i < nCount && !pShapesMenu; ++i ) {
			CString str;
			pMenu->GetMenuString(i, str, MF_BYPOSITION);
			if ( str == "Shapes" )
				pShapesMenu = pMenu->GetSubMenu(i);
		}
		if ( pShapesMenu ) {

			unsigned int DeleteByID[] = {
				ID_SHAPES_INTERPOLATINGLINEARSWEEP, ID_SHAPES_SCHMIDTUNION,
				ID_SHAPES_VARIATIONALWARP, ID_SHAPES_VARDISPLACEMENTMAP, 
				ID_SHAPES_TESTLINEARSWEEPWARP, ID_SHAPES_PARAMETRICTOIMPLICITSWEEP };

			for ( unsigned int i = 0; i < sizeof(DeleteByID) / sizeof(unsigned int); ++i  ) {
				BOOL bResult = pMenu->DeleteMenu(DeleteByID[i], MF_BYCOMMAND);
				if (!bResult)
					MessageBox("Could not frob the farber!\n");
			}

			const char * DeleteByName[] = { "RMS Medusa", "Contour Stack" };
			for ( unsigned int i = 0; i < sizeof(DeleteByName) / sizeof(const char *); ++i  ) {
				_RMSInfo("MOOLATTE!\n");

				// find menu
				unsigned int nCount = pShapesMenu->GetMenuItemCount();
				int nFound = -1;
				for ( unsigned int j = 0; j < nCount && nFound < 0; ++j ) {
					CString str;
					pShapesMenu->GetMenuString(j, str, MF_BYPOSITION);
					if ( str == DeleteByName[i] )
						nFound = j;
				}
				if ( !nFound ) 
					MessageBox("Could not grob the garber!\n");
				BOOL bResult = pShapesMenu->DeleteMenu( nFound, MF_BYPOSITION );
				if (!bResult)
					MessageBox("Could not trob the tarber!\n");
			}

			m_bMenusUpdated = true;
		}

	}
}



void CStandard3DView::OnCreateGL()
{
	CGLView::OnCreateGL();

	// initialize child windows
	m_pChildWindows = new C3DViewChildWindowSet(this);

	// initialize Wml rendering system...
	m_pCamera = new Wml::ExtendedWmlCamera(1.0,1.0);

	// set default frame (lookat)
	m_pCamera->SetTargetFrame( Wml::Vector3f(0.0, 0.0, 5.0f), 
							   Wml::Vector3f(-1.0f, 0.0, 0.0),
							   Wml::Vector3f(0.0, 1.0f, 0.0), 
							   Wml::Vector3f(0.0, 0.0, 0.0));

	// create view controller
	m_pViewController = new CViewController(m_pCamera);
	m_pViewController->RegisterListener(this);

	// enable color material
	glEnable(GL_COLOR_MATERIAL);
	glColorMaterial(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE);
//	float vSpecular[4] = {1,1,1,1};
//	glMaterialfv( GL_FRONT_AND_BACK, GL_SPECULAR, vSpecular );
//	glMaterialf( GL_FRONT_AND_BACK, GL_SHININESS , 40.0f );

	// initialize depth test
	glEnable(GL_DEPTH_TEST);

	m_pToolManager->SetCamera( m_pCamera );

	CRect rect(0,0,0,0);
	m_pSceneManager->Create(NULL, "This Window Should Not Be Visible",
		0, rect, this, 1234);
	m_pSceneManager->SetTool3DManager( m_pToolManager );
	m_pToolManager->AddListener( m_pSceneManager );

	m_pSketchManager->Create(NULL, "Invisible CSketchManager", 0, rect, this, 12435);
	m_pRenderManager->Create(NULL, "Invisible CRenderManager", 0, rect, this, 12436);
	m_pSurfaceManager->Create(NULL, "Invisible CSurfaceManager", 0, rect, this, 12437);
	m_pAnimationManager->Create(NULL, "Invisible CAnimationManager", 0, rect, this, 12438);

	m_pSceneManager->RegisterChangeListener( this );
	m_pSceneManager->RegisterChangeListener( CParametersDialogBar::Get() );
	m_pSceneManager->RegisterChangeListener( CBlobTreeEditorDialogBar::Get() );
	m_pSceneManager->RegisterChangeListener( m_pRenderManager );
	m_pSceneManager->RegisterChangeListener( m_pRenderManager->GetTextureManager() );
	m_pSceneManager->RegisterChangeListener( m_pSketchManager );
	m_pSceneManager->RegisterChangeListener( m_pSurfaceManager );
	m_pSceneManager->RegisterChangeListener( ParameterEditor::Get() );

	m_pSketchManager->RegisterChangeListener( this );
	m_pSketchManager->SetCamera( m_pCamera );

	m_pRenderManager->RegisterChangeListener( this );
	m_pRenderManager->SetCamera( m_pCamera );

	// need to do this ASAP...
	m_pHUDManager->SetViewController( m_pViewController );

	m_pRenderManager->SetCurrentBlobTree( m_pSceneManager->GetThreadSafeCurrentBlobTreeField() );

	m_pRenderManager->OnCreateGL();

	// initialize surface manager w/ current BlobTree
	m_pSurfaceManager->AddListener( m_pRenderManager );
	m_pSurfaceManager->AddListener( m_pRenderManager->GetTextureManager() );
	m_pSurfaceManager->AddBlobTree( m_pSceneManager->GetCurrentBlobTree() );

	// register input stream listeners
//	rmsui::CInputEventStream::Get()->AddListener( &m_shadingOverlay );
	rmsui::CInputEventStream::Get()->AddListener( m_pToolManager );
	rmsui::CInputEventStream::Get()->AddListener( m_pHUDManager );
	rmsui::CInputEventStream::Get()->AddListener( m_pGestureEngine );
	rmsui::CInputEventStream::Get()->AddListener( m_pRenderManager->GetTextureManager() );

	// register gesture engine listeners
	m_pGestureEngine->AddListener( m_pHUDManager );
	m_pGestureEngine->AddListener( m_pRenderManager->GetTextureManager() );
	m_pGestureEngine->AddListener( m_pSceneManager );
	m_pGestureEngine->AddListener( m_pSketchManager );

	// register HUD toolbars
	m_pHUDManager->AddHUDToolbar( m_pSketchManager->GetSuggestionToolbar() );
	m_pHUDManager->AddHUDToolbar( m_pRenderManager->GetTextureManager()->GetTextureToolbar() );
	m_pHUDManager->AddHUDToolbar( ParameterEditor::Get()->GetSelectionParamToolbar() );
	m_pHUDManager->AddHUDToolbar( ParameterEditor::Get()->GetGlobalParamToolbar() );

	// register sketch interface components
	m_pSketchManager->RegisterSketchComponent( m_pRenderManager->GetTextureManager() );

	// register timer listeners and start timer
	m_timeGen.AddListener( m_pHUDManager );
	m_nTimeGenTimer = (unsigned int)SetTimer( (INT_PTR)1020, 75, NULL );
		
	// create background processor
	m_pProcessManagerThread = new CBackgroundProcessManager();
	m_pProcessManagerThread->CreateThread( CREATE_SUSPENDED );
	m_pProcessManagerThread->ResumeThread();

	// for releases...
	ConfigureDevelBuildFeatures();
}


void CStandard3DView::OnSizeGL(int cx, int cy)
{
	// set viewport
	m_pCamera->SetViewPort(0, 0, cx, cy);

	// set frustum
	m_pCamera->SetFrustum( 30.0f, float(cx)/float(cy), 0.1f, 100.0);

	// this forces a glViewport/glFrustum/glLookAt
	m_pViewController->UpdateCamera();

	// update stroke factory
	m_pSketchManager->DrawingAreaChanged( m_pCamera->GetViewport() );

	CString str;
	str.Format("Model View : %d x %d", cx, cy);
	GetParentFrame()->SetWindowText(str);

	m_shadingOverlay.NotifyViewportChanged( cx, cy );
}


void CStandard3DView::OnDrawGL()
{
	_RMSTUNE_start(1);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	m_pRenderManager->Render();

	// draw scene objects
	if (m_pSketchManager->GetSketchingEnabled() == false)
		m_pSceneManager->GetSceneObjectManager()->Render();

	// draw sketch manager stuff
	m_pSketchManager->Render( true );

	// draw shading overlay
//	m_shadingOverlay.Render(false);

	// enable gl normalize for tool manager...
	glEnable(GL_NORMALIZE);
	m_pToolManager->Render( );
	glDisable(GL_NORMALIZE);

	// render HUD
	if (m_bDrawHUD)
		m_pHUDManager->Render();

	if ( m_bAnimating) {

		// if we are saving file, grab the frame buffer...
		static const unsigned int nDupFrames = 1;
		if ( m_bCaptureFrames ) {
			rmsui::Image image;
			CaptureFrame(image);
			char buf[256];
			unsigned  int nCurFrame = (int)m_fCurTime;
			for ( unsigned int i = 0; i < nDupFrames; ++i ) {
				sprintf(buf, "frame%.4d.png", nCurFrame * nDupFrames + i);
				image.Save( CSystemConstants::GetAnimationPath() + CString(buf) );
			}
		}


		// step time
		m_fCurTime += 1.0f;
		m_pAnimationManager->SetCurrentFrame( (unsigned int)m_fCurTime );

#if 0
		// update polygonizer res
		m_pSurfaceManager->SetFixedResolution( 25 + (unsigned int)m_fCurTime );
#endif	

		CString str;
		str.Format("Frame %d of %d\n", m_pAnimationManager->GetCurrentFrame(),  m_pAnimationManager->GetMaxFrame() );
		( (CMainFrame *)AfxGetMainWnd() )->SetStatusBarText( str );

		Invalidate();

		// if we are at last frame, stop animation
		if ( (unsigned int)m_fCurTime >= m_pAnimationManager->GetMaxFrame() )
			m_bAnimating = false;
		if ( m_bStepOneFrameOnly ) {
			m_bAnimating = false;
			m_bStepOneFrameOnly = false;
		}

	}

	_RMSTUNE_end(1);
	float fTime = (float)_RMSTUNE_time(1);
	CString str;
	str.Format("%.2f FPS\n", 1.0f / fTime);
	( (CMainFrame *)AfxGetMainWnd() )->GetStatusBar().SetWindowText(str);
}


// CStandard3DView lighting





/*
 * CStandard3DView camera transformations
 */

void CStandard3DView::ZoomView( const CPoint & newPoint )
{
	int nDX = newPoint.x - m_ptLastMousePosition.x;
	int nDY = m_ptLastMousePosition.y - newPoint.y;

	m_pViewController->DollyZoom( (float)nDX * 0.07f, (float)nDY * 0.02f );
}

void CStandard3DView::PanView( const CPoint & newPoint )
{
	int nDX = newPoint.x - m_ptLastMousePosition.x;
	int nDY = newPoint.y - m_ptLastMousePosition.y;

	m_pViewController->Pan( (float)nDX * 0.01f, (float)nDY * 0.01f );
}

void CStandard3DView::OrbitView( const CPoint & newPoint )
{
	int nDX = newPoint.x - m_ptLastMousePosition.x;
	int nDY = m_ptLastMousePosition.y - newPoint.y;

	m_pViewController->Orbit( (float)nDX * 0.02f, (float)nDY * 0.02f );
}


void CStandard3DView::SmoothCameraTransition( const Wml::Vector3f & newTarget,
											  const Wml::Vector3f & newEye )
{

	m_cameraInterpolator = 	Wml::SmoothCameraInterpolator( m_pCamera, newEye, newTarget, 20 );
	m_nSmoothCameraTimer = (unsigned int)SetTimer( (INT_PTR)1010, 10, NULL );
}

void CStandard3DView::SmoothCameraTransition_TimerEvent()
{
	if ( m_cameraInterpolator.RunSlerpStep() == false )
		KillTimer( (INT_PTR)m_nSmoothCameraTimer );
	Invalidate();
}



/*
 *  CStandard3DView printing
 */

BOOL CStandard3DView::OnPreparePrinting(CPrintInfo* pInfo)
{
	return DoPreparePrinting(pInfo);
}
void CStandard3DView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
}
void CStandard3DView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
}



// CStandard3DView diagnostics

#ifdef _DEBUG
void CStandard3DView::AssertValid() const
{
	CView::AssertValid();
}
void CStandard3DView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}
CBlobShopDoc* CStandard3DView::GetDocument() const // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CBlobShopDoc)));
	return (CBlobShopDoc*)m_pDocument;
}
#endif //_DEBUG


// CStandard3DView message handlers


void CStandard3DView::OnSceneChanged()
{
	Invalidate();
}

void CStandard3DView::OnSketchChanged()
{
	Invalidate();
}

void CStandard3DView::OnTreeSelectionChanged( const std::vector<rmsui::SceneObject *> & newSelection )
{
	Invalidate();
}

void CStandard3DView::OnRenderParametersChanged()
{
	Invalidate();
}

void CStandard3DView::OnViewChanged()
{
	Invalidate();
}


/*
 * mouse handlers
 */

void CStandard3DView::UpdateMouseDown( UINT whichButton, bool bState )
{ 
	switch (whichButton) {
		case MK_LBUTTON:
			m_bLeftMouseDown = bState;
			break;
		case MK_MBUTTON:
			m_bMiddleMouseDown = bState;
			break;
		case MK_RBUTTON:
			m_bRightMouseDown = bState;
			break;
	}
}

void CStandard3DView::OnLButtonDown(UINT nFlags, CPoint point)
{
	if ( nFlags & MK_MBUTTON || nFlags & MK_RBUTTON)
		return;

	// could maybe avoid this if we acquire the Alt key state somehow in focus restoring (??)
	if ( ! m_bAltKeyDown ) {

		const unsigned int * pViewport = m_pCamera->GetViewport();
		Wml::Ray3f ray;
		m_pCamera->GetPickRay(point.x, point.y, pViewport[2], pViewport[3],
			ray.Origin(), ray.Direction() );

		// [RMS: scene object manager selection disabled. Do not want to select primitives this way.
		//   Will need to select children of primitives (like bezier control points), so this will 
		//   have to be re-enabled at some point. But that selection should happen internally as an
		//   input event listener...
		if ( m_pSketchManager->GetSketchingEnabled() == false &&
			 m_pSceneManager->GetSceneObjectManager()->TestForObjectHit(ray) == true ) {

			if ( m_pSceneManager->GetSceneObjectManager()->SelectObject(ray, ( (nFlags & MK_CONTROL) != 0) ) ) {

				// update tool manager
				m_pSceneManager->GetSceneObjectManager()->UpdateToolManager( m_pToolManager );
				m_pSceneManager->NotifyMousePickActionOccurred();
				m_pToolManager->EnableCurrentTool(true);
			}

		} else {

			// send un-consumed mouse event to input event stream
			rmsui::CInputEventStream::Get()->AppendLeftMouseDownEvent(point.x, m_pCamera->GetViewport()[3]-1-point.y);
		}

	} // end alt-key-down test


	Invalidate();

	UpdateMouseDown( MK_LBUTTON, true );
	CGLView::OnLButtonDown(nFlags, point);

	SetCapture();
}

void CStandard3DView::OnLButtonUp(UINT nFlags, CPoint point)
{
	if ( nFlags & MK_MBUTTON || nFlags & MK_RBUTTON)
		return;

	rmsui::CInputEventStream::Get()->AppendLeftMouseUpEvent(point.x, m_pCamera->GetViewport()[3]-1-point.y);

	// repaint ??
	Invalidate();

	// TODO: Add your message handler code here and/or call default

	UpdateMouseDown( MK_LBUTTON, false );
	CGLView::OnLButtonUp(nFlags, point);

	ReleaseCapture();
}

void CStandard3DView::OnLButtonDblClk(UINT nFlags, CPoint point)
{
	if ( nFlags & MK_MBUTTON || nFlags & MK_RBUTTON)
		return;

	// TODO: Add your message handler code here and/or call default
	CGLView::OnLButtonDblClk(nFlags, point);
}

void CStandard3DView::OnMButtonDown(UINT nFlags, CPoint point)
{
	if ( nFlags & MK_LBUTTON || nFlags & MK_RBUTTON)
		return;

	// TODO: Add your message handler code here and/or call default

	UpdateMouseDown( MK_MBUTTON, true );
	CGLView::OnMButtonDown(nFlags, point);

	SetCapture();
}

void CStandard3DView::OnMButtonUp(UINT nFlags, CPoint point)
{
	if ( nFlags & MK_LBUTTON || nFlags & MK_RBUTTON)
		return;

	// TODO: Add your message handler code here and/or call default

	UpdateMouseDown( MK_MBUTTON, false );
	CGLView::OnMButtonUp(nFlags, point);

	ReleaseCapture();
}

void CStandard3DView::OnMButtonDblClk(UINT nFlags, CPoint point)
{
	if ( nFlags & MK_LBUTTON || nFlags & MK_RBUTTON)
		return;

	// TODO: Add your message handler code here and/or call default

	CGLView::OnMButtonDblClk(nFlags, point);
}

void CStandard3DView::OnRButtonDown(UINT nFlags, CPoint point)
{
	if ( nFlags & MK_LBUTTON || nFlags & MK_MBUTTON)
		return;

	Invalidate();

	UpdateMouseDown( MK_RBUTTON, true );
	CGLView::OnRButtonDown(nFlags, point);

	rmsui::CInputEventStream::Get()->AppendRightMouseDownEvent(point.x, m_pCamera->GetViewport()[3]-1-point.y);

	SetCapture();
}

void CStandard3DView::OnRButtonUp(UINT nFlags, CPoint point)
{
	if ( nFlags & MK_LBUTTON || nFlags & MK_MBUTTON)
		return;

	// TODO: Add your message handler code here and/or call default

	UpdateMouseDown( MK_RBUTTON, false );
	CGLView::OnRButtonUp(nFlags, point);

	rmsui::CInputEventStream::Get()->AppendRightMouseUpEvent(point.x, m_pCamera->GetViewport()[3]-1-point.y);

	ReleaseCapture();
}

void CStandard3DView::OnRButtonDblClk(UINT nFlags, CPoint point)
{
	if ( nFlags & MK_LBUTTON || nFlags & MK_MBUTTON)
		return;

	// TODO: Add your message handler code here and/or call default

	CGLView::OnRButtonDblClk(nFlags, point);
}

void CStandard3DView::OnMouseMove(UINT nFlags, CPoint point)
{
	// if alt is down, we do camera movement
	if (m_bAltKeyDown) {
		if (m_bLeftMouseDown)
			OrbitView(point);
		else if (m_bMiddleMouseDown) 
			PanView(point);
		else if (m_bRightMouseDown)
			ZoomView(point);

	} else {
		if ( m_bLeftMouseDown )
			rmsui::CInputEventStream::Get()->AppendLeftMouseMoveEvent(point.x, m_pCamera->GetViewport()[3]-1-point.y);
		else if ( m_bRightMouseDown ) {
			rmsui::CInputEventStream::Get()->AppendRightMouseMoveEvent(point.x, m_pCamera->GetViewport()[3]-1-point.y);
			Invalidate();
		}

		// draw current stroke w/o repaint
		BeginGLCalls();
		glDrawBuffer(GL_FRONT_AND_BACK);
		m_pGestureEngine->RenderCurrentStroke( m_pCamera->GetViewport()[2], m_pCamera->GetViewport()[3] );
		glDrawBuffer(GL_BACK);
		EndGLCalls();

#if 0
		BeginGLOverlayCalls();
//		BeginGLCalls();
		glDrawBuffer(GL_FRONT_AND_BACK);
//		glClearColor(1,0,0,1);
//		glClear(GL_COLOR_BUFFER_BIT);
		m_shadingOverlay.Render(true);
		SwapOverlayBuffers();
		EndGLOverlayCalls();
#endif
	}


	UpdateMousePosition( point );
	CGLView::OnMouseMove(nFlags, point);
}







/*
 * keyboard messages
 */

#include <DebugUtils.h>
void CStandard3DView::OnChar(UINT nChar, UINT nRepCnt, UINT nFlags)
{
	BeginGLCalls();
	
	if (nChar == 'c') {
		rmsbt::FieldCacheManager::Get()->EnableCaching( 
				! rmsbt::FieldCacheManager::Get()->IsCachingEnabled() );		
		CString str;
		str.Format("Caching is %d\n", rmsbt::FieldCacheManager::Get()->IsCachingEnabled() );
		OutputDebugString(str);
		m_pRenderManager->OnSceneChanged();
	} if (nChar == 'e') {
		float fCamDist = (m_pCamera->GetTarget() - m_pCamera->GetLocation()).Length();
		Wml::Vector3f dir(1.0, 1.0, 0.0);
		dir.Normalize();
		SmoothCameraTransition( 
			m_pCamera->GetTarget() + Wml::Vector3f(0.5f, 0.0f, 0.0f), 
			dir * -fCamDist * 2.0f );

	} else if (nChar == 't') {
		m_pToolManager->SelectNextValidTool();
	} else if (nChar == 'i') {
		m_pRenderManager->OnSceneChanged();
	}  else if (nChar == 'r') {
		if (m_pChildWindows) {
			Wml::AxisAlignedBox3f bounds;
			m_pSceneManager->GetThreadSafeCurrentBlobTreeField()->FieldBounds(bounds);
			m_pChildWindows->GetFieldImageView()->UpdateField( m_pSceneManager->GetThreadSafeCurrentBlobTreeField(), bounds );
		}

	} else if (nChar == 'z') {
		if ( m_pChildWindows ) {
			m_pChildWindows->GetRayTracedView()->UpdateImage(
				m_pSceneManager->GetThreadSafeCurrentBlobTreeField(), 
				m_pViewController->GetCamera() );
		}
	} else if (nChar == '+') {
		m_pSurfaceManager->IncreaseSurfacingAccuracy( 0.9f, true );
	} else if (nChar == '-') {
		m_pSurfaceManager->DecreaseSurfacingAccuracy( 1.1f, true );
	} else if (nChar == 'd') {
//		m_pToolManager->Translate( Wml::Vector3f(0, 0, -0.1f), true );
//		m_pSceneManager->NotifyToolActionOcurred();
//		m_pRenderManager->OnSceneChanged();
	} else if (nChar == 'u') {
//		m_pToolManager->Translate( Wml::Vector3f(0, 0, 0.1f), true );
//		m_pSceneManager->NotifyToolActionOcurred();
//		m_pRenderManager->OnSceneChanged();
	}  else if (nChar == 's') {
		GetParentFrame()->SetWindowPos(NULL, 0, 0, 652, 511, SWP_NOMOVE | SWP_NOZORDER);
	} else if (nChar == 'p') {
		m_pRenderManager->GetTextureManager()->CycleSelectedParamType();
	} else if (nChar == '1') {
		m_pRenderManager->GetTextureManager()->SetSelectedPatchType( rmsui_tex::SurfacePatchSceneObject::ExponentialMapPatch );
	} else if ( nChar == '2') {
		m_pRenderManager->GetTextureManager()->SetSelectedPatchType( rmsui_tex::SurfacePatchSceneObject::HybridPatch );
	} else if ( nChar == '3') {
		m_pRenderManager->GetTextureManager()->SetSelectedPatchType( rmsui_tex::SurfacePatchSceneObject::NaturalConformalPatch );
	} else if (nChar == 'a') {
		m_pRenderManager->GetTextureManager()->OnTextureSettingsAddPatchButton();
	} else if  (nChar == '=' ) {

	} else if (nChar == '\\' ) {

		//Wml::Vector3f vLocation = m_pViewController->GetCamera()->GetLocation();
		//Wml::Vector3f vDirection = m_pViewController->GetCamera()->GetDirection();
		//Wml::Vector3f vUp = m_pViewController->GetCamera()->GetUp();

		//rmsimplicit::Point p;
		//rmsimplicit::DebugUtils::PrintDerivInfo(&p, 0.1f, 0.12f, 0.13f);

		//m_pRenderManager->ExternalHackFunc();

//		rmsui::Image tmpImage(128, 128, 4);
//		tmpImage.SetCompositeMode( rmsui::Image::Accumulate );
//		tmpImage.Clear( Wml::ColorRGBA::BLACK );
//		//tmpImage.DrawLine( 50, 50, 100, 55, Wml::ColorRGBA::WHITE, 16.0f );
//		tmpImage.DrawDiscAA( 60, 60, 51, Wml::ColorRGBA(0.5f,0.5f,0.5f,1) );
////		tmpImage.DrawCircle( 60, 60, 51, Wml::ColorRGBA(0.2f,0.2f,0.2f,1) );
//		tmpImage.Save("c:\\temp\\linetest1.png");
//		tmpImage.Clear( Wml::ColorRGBA::BLACK );
//		tmpImage.DrawDiscAA( 60, 60, 50, Wml::ColorRGBA(0.5f,0.5f,0.5f,1) );
////		tmpImage.DrawCircle( 60, 60, 50, Wml::ColorRGBA(0.2f,0.2f,0.2f,1) );
//		tmpImage.Save("c:\\temp\\linetest2.png");
//		tmpImage.Clear( Wml::ColorRGBA::BLACK );
//		tmpImage.DrawDiscAA( 60, 61, 50, Wml::ColorRGBA(0.5f,0.5f,0.5f,1) );
////		tmpImage.DrawCircle( 60, 61, 50, Wml::ColorRGBA(0.2f,0.2f,0.2f,1) );
//		tmpImage.Save("c:\\temp\\linetest3.png");

		//m_pRenderManager->GetTextureManager()->RunGeodesicTest();
		//m_pRenderManager->GetTextureManager()->RunParamTest();
		//m_pRenderManager->GetTextureManager()->RunSphereTests(
		//	"C:\\temp\\exptest\\sphere_mc_500.obj", "C:\\temp\\exptest\\REV_sphere_mc_500" );
		//m_pRenderManager->GetTextureManager()->RunSphereTests(
		//	"C:\\temp\\exptest\\sphere_mc_5000.obj", "C:\\temp\\exptest\\REV_sphere_mc_5000" );
		//m_pRenderManager->GetTextureManager()->RunSphereTests(
		//	"C:\\temp\\exptest\\sphere_ref_500.obj", "C:\\temp\\exptest\\REV_sphere_ref_500" );
		//m_pRenderManager->GetTextureManager()->RunSphereTests(
		//	"C:\\temp\\exptest\\sphere_ref_5000.obj", "C:\\temp\\exptest\\REV_sphere_ref_5000" );

	}

	EndGLCalls();

	Invalidate();

	CGLView::OnChar(nChar, nRepCnt, nFlags);
}


void CStandard3DView::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags)
{
	if ( nChar == VK_ESCAPE )
		OnAnimationStop();
	CGLView::OnKeyDown(nChar, nRepCnt, nFlags);
}


void CStandard3DView::OnKeyUp(UINT nChar, UINT nRepCnt, UINT nFlags)
{
	// TODO: Add your message handler code here and/or call default

	CGLView::OnKeyUp(nChar, nRepCnt, nFlags);
}

void CStandard3DView::OnSysKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags)
{
	if (nChar == VK_MENU)		// this is the ALT key
		m_bAltKeyDown = true;

	CGLView::OnSysKeyDown(nChar, nRepCnt, nFlags);
}

void CStandard3DView::OnSysKeyUp(UINT nChar, UINT nRepCnt, UINT nFlags)
{
	if (nChar == VK_MENU)		// this is the ALT key
		m_bAltKeyDown = false;

	CGLView::OnSysKeyUp(nChar, nRepCnt, nFlags);
}


void CStandard3DView::OnViewFieldimage()
{
	m_pChildWindows->ShowFieldImageView();
}


void CStandard3DView::OnUpdateViewFieldimage(CCmdUI *pCmdUI)
{
	pCmdUI->SetCheck( m_pChildWindows->IsFieldImageVisible() == true );
}


void CStandard3DView::OnWindowRaytrace()
{
	m_pChildWindows->ShowRayTracedView();
}



void CStandard3DView::OnTimer(UINT nIDEvent)
{
	if (nIDEvent == m_nSmoothCameraTimer) {
		SmoothCameraTransition_TimerEvent();
	} else if ( nIDEvent == m_nTimeGenTimer ) {
		if ( m_timeGen.Tick() ) {
			Invalidate();
		}
	}

	CGLView::OnTimer(nIDEvent);
}



void CStandard3DView::OnKillFocus(CWnd* pNewWnd)
{
	CGLView::OnKillFocus(pNewWnd);
	if (m_bLeftMouseDown)
		OnLButtonUp(0, m_ptLastMousePosition);
	if (m_bRightMouseDown)
		OnRButtonUp(0, m_ptLastMousePosition);
	if (m_bMiddleMouseDown)
		OnMButtonUp(0, m_ptLastMousePosition);
}

void CStandard3DView::OnSetFocus(CWnd* pOldWnd)
{
	CGLView::OnSetFocus(pOldWnd);

	// do menu stuff (this sort of works...)
	ConfigureDevelBuildFeatures();

	// set current input event stream
	rmsui::CInputEventStream::SetCurrentEventStream(this);

	// set current scene manager
	CSceneManager::SetCurrentSceneManager(this);
	m_pSceneManager = CSceneManager::Get();

	if ( (GetAsyncKeyState(VK_MENU) & 0x8000) != 0 )
		m_bAltKeyDown = true;
	else
		m_bAltKeyDown = false;

	CBlobTreeEditorDialogBar::Get()->SetCurrentSceneManager( m_pSceneManager );
}


void CStandard3DView::OnViewEnablesketching()
{
	m_pSketchManager->SetSketchingEnabled( ! m_pSketchManager->GetSketchingEnabled() );
}

void CStandard3DView::OnUpdateViewEnablesketching(CCmdUI *pCmdUI)
{
	pCmdUI->SetCheck( (m_pSketchManager->GetSketchingEnabled() == true) );
}



BOOL CStandard3DView::OnCmdMsg(UINT nID, int nCode, void* pExtra, AFX_CMDHANDLERINFO* pHandlerInfo)
{
	// override default command message routing to pass messages to our managers...

	// give managers a chance
	if (m_pSketchManager->OnCmdMsg(nID, nCode, pExtra, pHandlerInfo))
		return TRUE;
	if (m_pSceneManager->OnCmdMsg(nID, nCode, pExtra, pHandlerInfo))
		return TRUE;
	if (m_pRenderManager->OnCmdMsg(nID, nCode, pExtra, pHandlerInfo))
		return TRUE;
	if (m_pSurfaceManager->OnCmdMsg(nID, nCode, pExtra, pHandlerInfo))
		return TRUE;
	if (m_pAnimationManager->OnCmdMsg(nID, nCode, pExtra, pHandlerInfo))
		return TRUE;

	return __super::OnCmdMsg(nID, nCode, pExtra, pHandlerInfo);
}


void CStandard3DView::OnViewCachingcontrol()
{
	CCachingControlDialog dialog;

	bool bCachingEnabled = rmsbt::FieldCacheManager::Get()->IsCachingEnabled();
	dialog.m_bCachingEnabledCheck = bCachingEnabled;

	INT_PTR nRet = -1;
	nRet = dialog.DoModal();

	if (nRet == IDOK) {
		if ( dialog.m_bCachingEnabledCheck != bCachingEnabled)
			rmsbt::FieldCacheManager::Get()->EnableCaching( dialog.m_bCachingEnabledCheck );
		m_pRenderManager->OnSceneChanged();
	}
}

void CStandard3DView::OnStatisticsFieldvalueapproximationerror()
{
	m_pRenderManager->ComputeStatistics( CRenderingManager::FieldValueErrorAtMeshVertices, "c:\\rmsbt_field_value_error.txt" );
}

void CStandard3DView::OnStatisticsCachememoryusage()
{
	m_pRenderManager->ComputeStatistics( CRenderingManager::CacheMemoryUsage, "c:\\rmsbt_cache_memory.txt");
}

void CStandard3DView::OnStatisticsCacheevaluation()
{
	m_pRenderManager->ComputeStatistics( CRenderingManager::CacheEvaluations, "c:\\rmsbt_cache_evals.txt");
}


void CStandard3DView::OnFileSave()
{
	Wml::Serializer * pSerializer = new Wml::Serializer();

	// [RMS: currently these records have "known" id's. But is that really necessary? can't we just
	//   store their ID's in some meta-record? Then we would only need one known record for the whole system...

	// save blobtrees
	BlobTreeListRecord BTList;
	rmsui_bt::BlobTreeSceneObject * pCurTree = m_pSceneManager->GetCurrentBlobTree();
	Wml::SerializerRecord * pBTRecord = pCurTree->GetRootNode()->Serialize(pSerializer);
	BTList.AddBlobTreeRecord( pBTRecord );
	BTList.Serialize( pSerializer );

	// save texture manager
	TextureManagerRecord pTexManRec;
	Wml::SerializerRecord * pTexManData = m_pRenderManager->GetTextureManager()->Serialize( pSerializer );
	pTexManRec.SetTextureManagerRecord( pTexManData );
	pTexManRec.Serialize( pSerializer );

	// save optoins
	SceneOptionsRecord optRec;
	Wml::SerializerRecord * pViewData = m_pViewController->Serialize( pSerializer );
	optRec.SetViewRecord( pViewData );
	optRec.Serialize( pSerializer );

	CFileDialog dialog(FALSE, "ssn", "new.ssn", OFN_OVERWRITEPROMPT, "Scene Files (*.ssn)|*.ssn|All Files (*.*)|*.*", this);
	if ( dialog.DoModal() == IDOK ) {
		CString pathName = dialog.GetPathName();
		if (! pSerializer->Write( (LPCSTR)pathName ) )
			MessageBox("Error Saving File", "Error", MB_OK);
	}

	delete pSerializer;
}


void CStandard3DView::OnFileExportselection()
{
	Wml::Serializer * pSerializer = new Wml::Serializer();

	BlobTreeListRecord * pBTList = new BlobTreeListRecord();

	Wml::SerializerRecord * pBTRecord = m_pSceneManager->SerializeSelection(pSerializer);
	
	if ( pBTRecord ) {
		pBTList->AddBlobTreeRecord( pBTRecord );

		pBTList->Serialize( pSerializer );

		CFileDialog dialog(FALSE, "ssn", "new.ssn", OFN_OVERWRITEPROMPT, "Scene Files (*.ssn)|*.ssn|All Files (*.*)|*.*", this);
		if ( dialog.DoModal() == IDOK ) {
			CString pathName = dialog.GetPathName();
			if (! pSerializer->Write( (LPCSTR)pathName ) )
				MessageBox("Error Saving File", "Error", MB_OK);
		}
	} else {
		MessageBox("Error Saving File", "Error", MB_OK);
	}

	delete pBTList;
	delete pSerializer;
}
void CStandard3DView::OnUpdateFileExportselection(CCmdUI *pCmdUI)  { pCmdUI->Enable( m_pSceneManager->HasSelection() ); }



void CStandard3DView::OnFileOpen()
{
	Wml::Serializer * pSerializer = NULL;

	// load serializer from file
		
	CFileDialog dialog(TRUE, "ssn", "*.ssn", 0, "Scene Files (*.ssn)|*.ssn|All Files (*.*)|*.*", this);
	if ( dialog.DoModal() == IDOK ) {
		CString pathName = dialog.GetPathName();
		pSerializer = new Wml::Serializer();
		if (! pSerializer->Read( (LPCSTR)pathName ) ) {
			delete pSerializer;
			pSerializer = NULL;
			MessageBox("Error Loading File", "Error", MB_OK);
		}
	}
	if ( pSerializer == NULL )
		return;

	// create scene reader
	SceneFileReader SceneReader( m_pSceneManager, m_pRenderManager->GetTextureManager(), m_pViewController );

	if ( SceneReader.RestoreScene( pSerializer ) == false )
		MessageBox("Error Loading Scene", "Error", MB_OK);
}



void CStandard3DView::OnExportmeshObjformat()
{
	std::vector< IsoSurfaceMesh * > vMeshes;
	m_pSurfaceManager->GetSurfaceList( vMeshes );
	size_t nCount = vMeshes.size();

	rmsimplicit::TriangleMesh * pMesh = NULL;
	for ( unsigned int i = 0; i < nCount && pMesh == NULL; ++i )
		if ( vMeshes[i]->GetPolygonizerType() == IsoSurfaceMesh::StaticMC )
			pMesh = vMeshes[i]->GetSimpleMesh();
	if (!pMesh)
		return;

	CFileDialog dialog(FALSE, "obj", "mesh.obj", OFN_OVERWRITEPROMPT, "OBJ Files (*.obj)|*.obj|All Files (*.*)|*.*", this);
	if( dialog.DoModal ()==IDOK ) {
		CString pathName = dialog.GetPathName();
		pMesh->write( (LPCSTR)pathName, rmsimplicit::TriangleMesh::OBJ_FORMAT );
	}
}


void CStandard3DView::OnExportmeshMeshliteformat()
{
	std::vector< IsoSurfaceMesh * > vMeshes;
	m_pSurfaceManager->GetSurfaceList( vMeshes );
	size_t nCount = vMeshes.size();

	rmsimplicit::TriangleMesh * pMesh = NULL;
	for ( unsigned int i = 0; i < nCount && pMesh == NULL; ++i )
		if ( vMeshes[i]->GetPolygonizerType() == IsoSurfaceMesh::StaticMC )
			pMesh = vMeshes[i]->GetSimpleMesh();
	if (!pMesh)
		return;

	CFileDialog dialog(FALSE, "m", "mesh.m", OFN_OVERWRITEPROMPT, "MeshLite Files (*.m)|*.m|All Files (*.*)|*.*", this);
	if( dialog.DoModal ()==IDOK ) {
		CString pathName = dialog.GetPathName();
		pMesh->write( (LPCSTR)pathName, rmsimplicit::TriangleMesh::MESHLITE_FORMAT );
	}
}



void CStandard3DView::OnEditRefinemesh()
{
	CMeshRefinementOptionsDialog optDialog;
	if (optDialog.DoModal() == IDOK) {
		m_pSurfaceManager->RefineAllMeshes( optDialog.nRZNSteps, optDialog.nZNSteps, optDialog.nRNSteps, 
			optDialog.nRSteps, optDialog.nZSteps, optDialog.nNSteps, optDialog.bSubdivideOnce );
	}
}



void CStandard3DView::OnEditQuickrefine()
{
#ifndef _DEBUG
	m_pSurfaceManager->RefineAllMeshes( 0, 0, 0, 5, 3, 0, false );
#else
	m_pSurfaceManager->RefineAllMeshes( 0, 0, 0, 5, 3, 0, false );
#endif
}



void CStandard3DView::OnEditIncreaseMeshRes()
{
	//m_pRenderManager->IncreaseSurfacingAccuracy( 0.9f, true );
	//Invalidate();
}

void CStandard3DView::OnEditDecreaseMeshRes()
{
	//m_pRenderManager->DecreaseSurfacingAccuracy( 1.1f, true );
	//Invalidate();
}

void CStandard3DView::OnEditCycletool()
{
	m_pToolManager->SelectNextValidTool();
	Invalidate();
}

void CStandard3DView::OnViewPerspectivecamera()
{
	bool bUsePerspective = m_pCamera->GetUsePerspective();
	m_pCamera->SetUsePerspective(! bUsePerspective );
	Invalidate();
}

void CStandard3DView::OnUpdateViewPerspectivecamera(CCmdUI *pCmdUI)
{
	pCmdUI->SetCheck( m_pCamera->GetUsePerspective() == true );
}

void CStandard3DView::OnEditClearcutplane()
{
	m_pSceneManager->ClearTemporaryCuttingPlane();
	Invalidate();
}

void CStandard3DView::OnEditTogglecaching()
{
	rmsbt::FieldCacheManager::Get()->EnableCaching( 
		! rmsbt::FieldCacheManager::Get()->IsCachingEnabled() );
	m_pRenderManager->OnSceneChanged();
}

void CStandard3DView::OnUpdateEditTogglecaching(CCmdUI *pCmdUI)
{
	pCmdUI->SetCheck( rmsbt::FieldCacheManager::Get()->IsCachingEnabled() );
}

void CStandard3DView::OnViewdirectionsLeft() { m_pViewController->ViewLeft(); }
void CStandard3DView::OnViewdirectionsRight() { m_pViewController->ViewRight(); }
void CStandard3DView::OnViewdirectionsTop() { m_pViewController->ViewTop(); }
void CStandard3DView::OnViewdirectionsBottom() { m_pViewController->ViewBottom(); }
void CStandard3DView::OnViewdirectionsFront() { m_pViewController->ViewFront(); }
void CStandard3DView::OnViewdirectionsBack() { m_pViewController->ViewBack(); }
void CStandard3DView::OnViewdirectionsThreequarter() { m_pViewController->ViewThreeQuarter(); }


void CStandard3DView::OnAnimationPlay()
{
	// set to frame 0
	if ( m_pAnimationManager->GetCurrentFrame() == m_pAnimationManager->GetMaxFrame() )
		m_pAnimationManager->SetCurrentFrame(0);

	// ok, now we want to render and then step to the next frame. Cannot force
	//  render, so we set a flag to cause a new frame at the end of teh render
	m_bAnimating = true;
	m_fCurTime = (float)m_pAnimationManager->GetCurrentFrame();

	Invalidate();
}



void CStandard3DView::OnAnimationPlayoneframe()
{
	m_bStepOneFrameOnly = true;
	OnAnimationPlay();
}


void CStandard3DView::OnAnimationStop()
{
	m_bAnimating = false;
}



void CStandard3DView::OnAnimationRewind()
{
	m_bAnimating = false;
	m_pAnimationManager->SetCurrentFrame(0);
	Invalidate();
}

void CStandard3DView::OnViewShowhud()  { m_bDrawHUD = ! m_bDrawHUD; Invalidate(); }
void CStandard3DView::OnUpdateViewShowhud(CCmdUI *pCmdUI) { pCmdUI->SetCheck( m_bDrawHUD ); }


void CStandard3DView::CaptureFrame(rmsui::Image & image)
{
	// get viewport
	const unsigned int * pViewport = m_pCamera->GetViewport();


	// make buffer
	std::vector<unsigned char> vBuffer;
	vBuffer.resize( pViewport[2] * pViewport[3] * 3 );		// not reading alpha...
	glReadBuffer(GL_BACK);
	glPixelStorei(GL_PACK_ALIGNMENT, 1);
	glReadPixels( 0, 0, pViewport[2], pViewport[3], GL_RGB, GL_UNSIGNED_BYTE, &vBuffer[0] );
	glReadBuffer(GL_FRONT);

	// init image
	image.InitFromBuffer( pViewport[2], pViewport[3], 3, &vBuffer[0], rmsui::Image::RGB_CHANNEL_MAP, false );

	image.Flip(false, true);
}

void CStandard3DView::OnAnimationSaveframes() { m_bCaptureFrames = ! m_bCaptureFrames; }
void CStandard3DView::OnUpdateAnimationSaveframes(CCmdUI *pCmdUI) { pCmdUI->SetCheck(m_bCaptureFrames); }

