// Ryan Schmidt - rms@unknownroad.com
// Copyright (c) 2005-2010
// Distributed under the Boost Software License, Version 1.0.
// http://www.boost.org/LICENSE_1_0.txt


#ifndef _RMSUI_HUD_STROKES_H_
#define _RMSUI_HUD_STROKES_H_

#include <string>
#include <set>
#include <vector>

#include <WmlAxisAlignedBox2.h>
#include <WmlBox2.h>
#include <WmlVector2.h>

#include "InputEventStream.h"
#include "GestureStrokes.h"

namespace rmsui
{

class IGestureEventListener
{
public:
	virtual ~IGestureEventListener() {}

	virtual bool OnPointStroke( GesturePointStroke * pPointStroke ) { return false; }
	virtual bool OnLineStroke( GestureLineStroke * pLineStroke ) { return false; }
	virtual bool OnVStroke( GestureVStroke * pVStroke ) { return false; }
	virtual bool OnScribbleStroke( GestureScribbleStroke * pScribbleStroke ) { return false; }
	virtual bool OnGeneralStroke( GestureStroke * pStroke) { return false; }
};


class GestureEngine
	: public IInputEventListener
{
public:
	GestureEngine();
	~GestureEngine();

	inline void AddListener( IGestureEventListener * pListener );
	inline void RemoveListener( IGestureEventListener * pListener );

	/* 
	 * IInputEvevntListener interface
	 */
	virtual EventState ProcessInputEvent( const InputEventData & event );

	/*
	 * gesture rendering
	 */
	void RenderCurrentStroke(unsigned int nScreenWidth, unsigned int nScreenHeight);

protected:
	std::set<IGestureEventListener *> m_vUniqueListeners;
	std::vector<IGestureEventListener *>m_vListeners;

	bool NotifyLineStroke( GestureLineStroke * pStroke );
	bool NotifyPointStroke( GesturePointStroke * pStroke );
	bool NotifyVStroke( GestureVStroke * pStroke );
	bool NotifyScribbleStroke( GestureScribbleStroke * pStroke );
	bool NotifyGeneralStroke( GestureStroke * pStroke );

	// stroke stuff

	rmsui::GestureStroke * m_pCurrentStroke;

	void StartStroke( const Wml::Vector2f & vStart );
	bool AppendToStroke( const Wml::Vector2f & vAppend );
	bool EndStroke( const Wml::Vector2f & vEnd );

	bool CheckCurrentStroke();
};



inline void GestureEngine::AddListener( IGestureEventListener * pListener )
{
//	m_vListeners.insert(pListener);
	if ( m_vUniqueListeners.find(pListener) == m_vUniqueListeners.end() ) {
		m_vUniqueListeners.insert( pListener );
		m_vListeners.push_back( pListener );
	}
}

inline void GestureEngine::RemoveListener( IGestureEventListener * pListener )
{
	ASSERT(0);
	//m_vListeners.erase(pListener);
}




}   // namespace rmsui



#endif // _RMSUI_HUD_STROKES_H_