// Ryan Schmidt - rms@unknownroad.com
// Copyright (c) 2005-2010
// Distributed under the Boost Software License, Version 1.0.
// http://www.boost.org/LICENSE_1_0.txt


#ifndef _RMSUI_HUD_IMAGE_H_
#define _RMSUI_HUD_IMAGE_H_

#include <vector>
#include <string>

#include "Texture.h"

namespace rmsui
{


class HUDIcon
{
public:
	virtual unsigned int Width() = 0;
	virtual unsigned int Height() = 0;

	virtual void Draw( const Wml::AxisAlignedBox2f & bounds ) = 0;
};


/*
 * Icon class that is loaded from file
 */
class HUDFileIcon : public HUDIcon
{
public:
	HUDFileIcon(const char * pFilename);
	virtual ~HUDFileIcon() {}

	virtual unsigned int Width() { return m_texture.Width(); }
	virtual unsigned int Height() { return m_texture.Height(); }

	virtual void Draw( const Wml::AxisAlignedBox2f & bounds );

protected:
	std::string m_filename;
	Texture m_texture;
};


/*
 * Icon class set from external image
 */
class HUDExternalImageIcon : public HUDIcon
{
public:
	HUDExternalImageIcon(Texture * pTexture);
	virtual ~HUDExternalImageIcon() {}

	virtual unsigned int Width() { return m_pTexture->Width(); }
	virtual unsigned int Height() { return m_pTexture->Height(); }

	virtual void Draw( const Wml::AxisAlignedBox2f & bounds );

protected:
	Texture * m_pTexture;
};


}   // namespace rmsui



#endif // _RMSUI_HUD_ITEM_H_