// Ryan Schmidt - rms@unknownroad.com
// Copyright (c) 2005-2010
// Distributed under the Boost Software License, Version 1.0.
// http://www.boost.org/LICENSE_1_0.txt


#ifndef __RMSUI_IMAGE_H__
#define __RMSUI_IMAGE_H__

#include <vector>
#include <WmlPolygon2.h>
#include <WmlColors.h>

namespace rmsui {

class Image {
public:
	Image(unsigned int nWidth = 0, unsigned int nHeight = 0, unsigned int nChannels = 4);
	Image( const Image & copy );

	unsigned int Height() const { return m_nHeight; }
	unsigned int Width() const { return m_nWidth; }
	unsigned int Channels() const { return m_nChannels; }

	const std::vector<unsigned char> & Buffer() const { return m_vBuffer; }

	void Crop( const Wml::Polygon2f & vPolygon, const Wml::ColorRGBA & vCropColor );

	bool Load( const char * pFilename );
	bool Save( const char * pFilename ) const;

	// rgb is channel map [0,1,2], bgr is channel map [2,1,0]
	void InitFromBuffer( unsigned int nBufferWidth, unsigned int nBufferHeight, unsigned int nBufferChannels,
						 const unsigned char * pBuffer,
						 const unsigned int nChannelMap[3], bool bAddAlpha );

	const unsigned char * GetPixel( unsigned int x, unsigned int y ) const;
	unsigned char * GetPixel( unsigned int x, unsigned int y );


	// these are used to map *to* RGB...
	static const unsigned int RGB_CHANNEL_MAP[3];
	static const unsigned int BGR_CHANNEL_MAP[3];
	static const unsigned int GRAY_CHANNEL_MAP[3];

	// simple image processing ops
	void AverageBlur( unsigned int nRadius );
	void RegionAverageBlur( const Wml::Polygon2f & vPolygon, unsigned int nRadius );

	void Clear( const Wml::ColorRGBA & cClearColor, bool bAlphaOnly = false );
	void Invert( );
	void Flip( bool bFlipX, bool bFlipY  );


	void ExpandChannels( unsigned int nNewNumChannels );


	void DrawLine( float fFromX, float fFromY, float fToX, float fToY, 
				   const Wml::ColorRGBA & cColor, float fWidth = 1.0f);
	void DrawCircle( float fCenterX, float fCenterY, float fRadius, const Wml::ColorRGBA & cColor );
	void DrawDisc( float fCenterX, float fCenterY, float fRadius, const Wml::ColorRGBA & cColor );
	void DrawDiscAA( float fCenterX, float fCenterY, float fRadius, const Wml::ColorRGBA & cColor );

	void FillQuad( const Wml::Vector2f vCorners[4], const Wml::ColorRGBA & cColor );
	void FillQuadAA( const Wml::Vector2f vCorners[4], const Wml::ColorRGBA & cColor );


	enum CompositeMode {
		Replace,
		Accumulate,
		Max
	};

	void SetCompositeMode( CompositeMode eMode ) { m_eMode = eMode; }
	CompositeMode GetCompositeMode() { return m_eMode; }
	void CompositePixel( unsigned int nX, unsigned int nY, const Wml::ColorRGBA & cColor, float fAlpha = 1.0f );


protected:
	unsigned int m_nWidth;
	unsigned int m_nHeight;

	unsigned int m_nChannels;

	CompositeMode m_eMode;

	std::vector<unsigned char> m_vBuffer;

	bool LoadLibPNG(const char * pFilename);
	bool LoadOpenCV(const char * pFilename);

	bool SaveOpenCV(const char * pFilename) const;


	void FillRow( int nXLow, int nXHigh, int nY, const Wml::ColorRGBA & cColor );
	void FillAADiscRow( int nXLow, int nXHigh, int nPadX, int nY, const Wml::ColorRGBA & cColor,
						float fX, float fY, float fRadius );
};

} // namespace rmsui


#endif // __RMSUI_IMAGE_FILE_H__