// Ryan Schmidt - rms@unknownroad.com
// Copyright (c) 2005-2010
// Distributed under the Boost Software License, Version 1.0.
// http://www.boost.org/LICENSE_1_0.txt


#ifndef _RMSUI_TIMER_H
#define _RMSUI_TIMER_H

#include <set>

namespace rmsui
{

typedef LARGE_INTEGER SystemTime;

class TimeEvent
{
public:
	TimeEvent();
	TimeEvent( SystemTime & timeInfo ) { m_time = timeInfo; }
	
	static TimeEvent Now();

	//! returns time delta fabsf(now - since)
	float SecondsDelta( const TimeEvent & time2 ) const;

protected:
	SystemTime m_time;
};


class ITimeEventListener
{
public:
	virtual ~ITimeEventListener() {}

	//! return is to be interpreted by caller...
	virtual bool OnTimeEvent( const TimeEvent & event ) = 0;
};



class TimeEventGenerator
{
public:
	TimeEventGenerator() {}
	~TimeEventGenerator() {}

	void AddListener( ITimeEventListener * pListener ) { m_vListeners.insert(pListener); }

	//! fire time events to listeners. If any listeners returns true, this function returns true (intepret it yourself!)
	bool Tick();

protected:
	std::set< ITimeEventListener * > m_vListeners;
};



};   // end namespace rmsui

#endif  // _RMSUI_TOOLSET_H_