// Ryan Schmidt - rms@unknownroad.com
// Copyright (c) 2005-2010
// Distributed under the Boost Software License, Version 1.0.
// http://www.boost.org/LICENSE_1_0.txt


#include "StdAfx.h"
#include "Tool.h"

using namespace rmsui;


void ITool3D::RegisterListener( ITool3DListener * pListener )
{
	m_vListeners.insert(pListener);
}

void ITool3D::NotifyToolActionOccured()
{
	std::set<ITool3DListener *>::iterator cur(m_vListeners.begin()), end(m_vListeners.end());
	while ( cur != end ) {
		(*cur++)->OnToolActionApplied(this);
	}
}



//! get the frame for this target
Wml::Frame ToolTargetGroup::GetFrame()
{
	// RMS TODO - anything else for this (??) Modes where we draw the frame at the middle of the points?
	//  rotation around a different reference point ?
	size_t nSize = m_vTargets.size();
	if ( nSize == 0 ) {
		return Wml::Frame3f();
	} else if ( nSize == 1 ) {
		return m_vTargets[0]->GetFrame();
	} else {
		float fScale = 1.0f / (float)nSize;
		Wml::Vector3f ptAvgOrigin( m_vTargets[0]->GetFrame().Origin() * fScale );
		for (unsigned int i = 1; i < nSize; ++i )
			ptAvgOrigin += m_vTargets[i]->GetFrame().Origin() * fScale;

		return Wml::Frame( ptAvgOrigin );
	}
}

//! translate the frame
void ToolTargetGroup::Translate( const Wml::Vector3f & vTranslate, bool bRelative )
{
	size_t nSize = m_vTargets.size();
	for (unsigned int i = 0; i < nSize; ++i) {
		m_vTargets[i]->Translate(vTranslate, bRelative);
	}
}

//! rotate the frame
void ToolTargetGroup::Rotate( const Wml::Matrix3f & rotation )
{
	size_t nSize = m_vTargets.size();
	for (unsigned int i = 0; i < nSize; ++i) {
		m_vTargets[i]->Rotate(rotation);
	}
}

//! test if target supports a tool
bool ToolTargetGroup::SupportsTool( const std::string & sToolName )
{
	// all selected objects must support tool...
	size_t nSize = m_vTargets.size();
	for (unsigned int i = 0; i < nSize; ++i) {
		if ( m_vTargets[i]->SupportsTool(sToolName) == false )
			return false;
	}
	return true;

}