// Ryan Schmidt - rms@unknownroad.com
// Copyright (c) 2005-2010
// Distributed under the Boost Software License, Version 1.0.
// http://www.boost.org/LICENSE_1_0.txt


#include "StdAfx.h"
#include <gl/GL.h>
#include <gl/GLU.h>
#include <WmlExtVectorUtil.h>
#include <WmlDistLin3Lin3.h>

#include "Tool3DRotation.h"

using namespace rmsui;

LocalRotationTool3D::LocalRotationTool3D( )
	: ITool3D("Rotation3D")
{
	float fRadius = 200.0;
	m_axisTargets[0] = Circle(Wml::Vector3f::ZERO, GetAxis(Wml::Frame::AxisX), fRadius );
	m_axisTargets[1] = Circle(Wml::Vector3f::ZERO, GetAxis(Wml::Frame::AxisY), fRadius );
	m_axisTargets[2] = Circle(Wml::Vector3f::ZERO, GetAxis(Wml::Frame::AxisZ), fRadius );
	m_tumbleTarget = Sphere(Wml::Vector3f::ZERO, fRadius);
	m_tumbleTargetOutline = Circle(Wml::Vector3f::ZERO,	Wml::Vector3f::UNIT_Z, fRadius);
	m_nSelected = 4;
}

void LocalRotationTool3D::TargetFrameChanged()
{	
	ITool3D::TargetFrameChanged();

	// RMS TODO this is not great - can't we just update the frame (??!) add Circle::SetFrame()
	float fRadius = 200.0;
	m_axisTargets[0] = Circle(Wml::Vector3f::ZERO, GetAxis(Wml::Frame::AxisX), fRadius );
	m_axisTargets[1] = Circle(Wml::Vector3f::ZERO, GetAxis(Wml::Frame::AxisY), fRadius );
	m_axisTargets[2] = Circle(Wml::Vector3f::ZERO, GetAxis(Wml::Frame::AxisZ), fRadius );
}


void LocalRotationTool3D::Render( const ViewState & viewState ) 
{
	//Basic3DFrameTool::Render( viewState );

	// RMS TODO TEMP REMOVE
	glPushAttrib(GL_ENABLE_BIT);
	glDisable(GL_LIGHTING);
	glPointSize(5.0);
	glColor3f(1.0, 1.0, 1.0);
	glBegin(GL_POINTS);
	glVertex3f(m_ptLinePoint.X(), m_ptLinePoint.Y(), m_ptLinePoint.Z());
	glEnd();
	glPopAttrib();

	// render tumble target into depth buffer to hide backs of lines
	glEnable(GL_POLYGON_OFFSET_FILL);
	glPolygonOffset(1.0, 0.0);
	glColorMask(GL_FALSE, GL_FALSE, GL_FALSE, GL_FALSE);
	m_tumbleTarget.Render();
	glColorMask(GL_TRUE, GL_TRUE, GL_TRUE, GL_TRUE);
	glDisable(GL_POLYGON_OFFSET_FILL);

	// render view-facing circle that contains tumble target
	// need to remove lookat for this... (this is sort of hacky)
	// then orient in lookat direction (??)
	glPushAttrib(GL_ENABLE_BIT);
	glDisable(GL_DEPTH_TEST);
	glPopMatrix();
	glPushMatrix();
	m_tumbleTargetOutline.Render( Wml::ColorRGBA(0.2f,0.2f,0.2f) );
	glMultMatrixf( &viewState.fCurrentTool_LookAt[0][0] );
	glPopAttrib();

	for (int i = 0; i < 3; ++i) {
		if ( i == m_nSelected )
			m_axisTargets[i].Render( Wml::ColorRGBA::BLACK );
		else
			m_axisTargets[i].Render( (i == 0) ? Wml::ColorRGBA::BLUE :
					(i == 1) ? Wml::ColorRGBA::RED : 
								Wml::ColorRGBA(0.0, 0.5f, 0.0));
	}
}



bool LocalRotationTool3D::Intersect( const Wml::Ray3f & ray, 
									 Wml::Vector3f & ptPosition, float & fDepth )
{
	float fNearestDepth = Wml::Mathf::MAX_REAL;
	Wml::Vector3f ptTemp;
	float fTempDepth;
	for (int i = 0; i < 3; ++i) {
		if (m_axisTargets[i].Intersect(ray, ptTemp, fTempDepth) && fTempDepth < fNearestDepth) {
			fNearestDepth = fTempDepth;
			ptPosition = ptTemp;
			m_nSelected = i;
		}
	}

	// try intersection w/ tumble target
	if (m_tumbleTarget.Intersect(ray, ptTemp, fTempDepth) 
		&& fTempDepth < fNearestDepth-40.0f ) {
		fNearestDepth = fTempDepth;
		ptPosition = ptTemp;
		m_nSelected = 4;
	}

	if (fNearestDepth != Wml::Mathf::MAX_REAL) {
		fDepth = fNearestDepth;
		return true;
	} else {
		return false;
	}
}


Wml::Vector3f LocalRotationTool3D::Move( SelectedTargetInfo & targetInfo,
										 const ViewState & viewState,
										 bool bInitialize )
{
	if (m_nSelected < 4) {
		return MoveAxis2( GetAxis((Wml::Frame::FrameAxis)m_nSelected ),
							targetInfo.fixedViewOrientedRay, targetInfo, bInitialize);
	} else if (m_nSelected == 4) {
		return MoveTumble(targetInfo, viewState, bInitialize);
	} else {
		ASSERT(0);
		return Wml::Vector3f::ZERO;
	}
}


Wml::Vector3f LocalRotationTool3D::MoveTumble( SelectedTargetInfo & targetInfo,
											   const ViewState & viewState,
											   bool bInitialize )
{
	// find intersection point on sphere
	Wml::Vector3f ptIntersection;
	float fTempDepth;
	if ( m_tumbleTarget.Intersect(targetInfo.fixedViewOrientedRay, ptIntersection, fTempDepth) ) {

		// RMS TODO : should these Origin always be Vector3f::ZERO (???)

		if (!bInitialize) {
			
			Wml::Vector3f v1( targetInfo.ptLastFrame - GetOrigin() );
			v1.Normalize();
			Wml::Vector3f v2( ptIntersection - GetOrigin() );
			v2.Normalize();
			Wml::Vector3f vAxis( v1.Cross(v2) );
			vAxis.Normalize();
			float fAcosAngle = (v1.Dot(v2)) / vAxis.Length();
			float fAngle = Wml::Mathf::ACos( fAcosAngle );
			Wml::Matrix3f rotation;
			rotation.FromAxisAngle( vAxis, fAngle );

			RotateFrame( rotation );
			UpdateCircleTargets();
		}

		targetInfo.ptLastFrame = ptIntersection;

		return Wml::Vector3f::ZERO;
	} else {
		
		// compute matrix that aligns Z axis with oriented ray direction
		Wml::Matrix3f matrix;
		Wml::ComputeAlignZAxisMatrix( targetInfo.fixedViewOrientedRay.Direction(), matrix );

		// move in plane w/ that axis as normal (plane is aligned w/ screen)
		return MoveAxis( matrix * Wml::Vector3f::UNIT_Z, 
			targetInfo.fixedViewOrientedRay, targetInfo, bInitialize );
	}
}



Wml::Vector3f LocalRotationTool3D::MoveAxis2( const Wml::Vector3f & vAxis,
											  const Wml::Ray3f & eyeRay,
											  SelectedTargetInfo & targetInfo,
											  bool bInitialize)
{
	Wml::Vector3f ptSphere, ptAxis;

	// find point on sphere
	float fTempDepth;
	if ( !  m_tumbleTarget.Intersect(targetInfo.fixedViewOrientedRay, ptSphere, fTempDepth)) {
		Wml::Line3f line;
		line.Origin() = targetInfo.fixedViewOrientedRay.Origin();
		line.Direction() = targetInfo.fixedViewOrientedRay.Direction();
		m_tumbleTarget.ClosestPoint(line, ptSphere);
	}

	// point on sphere is on non-oriented sphere, orient point using inverse of frame matrix
	ptSphere = GetFrame().FrameInverse() * ptSphere;

	float fRadius = m_tumbleTarget.Geometry().Radius();
	if (m_nSelected == 0) {
		float fTheta = Wml::Mathf::ATan2( ptSphere.Z() , ptSphere.Y() );
		Wml::Vector3f ptInitial( 0.0, Wml::Mathf::Cos(fTheta)*fRadius, Wml::Mathf::Sin(fTheta)*fRadius );
		ptAxis = GetFrame().FrameMatrix() * ptInitial;

	} else if (m_nSelected == 1) {
		float fTheta = Wml::Mathf::ATan2( ptSphere.X(), ptSphere.Z() );
		Wml::Vector3f ptInitial( Wml::Mathf::Sin(fTheta)*fRadius, 0.0, Wml::Mathf::Cos(fTheta)*fRadius  );
		ptAxis = GetFrame().FrameMatrix() * ptInitial;

	} else if (m_nSelected == 2) {
		float fTheta = Wml::Mathf::ATan2( ptSphere.Y() , ptSphere.X() );
		Wml::Vector3f ptInitial( Wml::Mathf::Cos(fTheta)*fRadius, Wml::Mathf::Sin(fTheta)*fRadius, 0.0  );
		ptAxis = GetFrame().FrameMatrix() * ptInitial;
	}

	if (!bInitialize) {

		// compute angle between new point and last point
		Wml::Vector3f v1( targetInfo.ptLastFrame );
		v1.Normalize();
		Wml::Vector3f v2( ptAxis );
		v2.Normalize();
		Wml::Vector3f vCrossAxis( v1.Cross(v2) );
		float fAngle = Wml::Mathf::ACos( v1.Dot(v2) );
		vCrossAxis.Normalize();

		Wml::Matrix3f rotation;
		rotation.FromAxisAngle(vCrossAxis, fAngle);

		RotateFrame(rotation);
		UpdateCircleTargets();
	}

	targetInfo.ptLastFrame = ptAxis;
	m_ptLinePoint = ptAxis;
	return Wml::Vector3f::ZERO;
}


Wml::Vector3f LocalRotationTool3D::MoveAxis( const Wml::Vector3f & vAxis,
											 const Wml::Ray3f & eyeRay,
											 SelectedTargetInfo & targetInfo,
											 bool bInitialize )
{
	// RMS TODO : should these Origin always be Vector3f::ZERO (???)

	// find intersection point between ray and selected plane
	const Wml::Vector3f & vPlaneNormal = vAxis;
	float fPlaneD = vPlaneNormal.Dot( GetOrigin() );

	float fDdotN = vPlaneNormal.Dot( eyeRay.Direction() );
	if (fabs(fDdotN) < Wml::Mathf::EPSILON)
		return GetOrigin();

	float fT = (fPlaneD - vPlaneNormal.Dot( eyeRay.Origin()) ) / fDdotN;
	Wml::Vector3f ptPlane( eyeRay.Origin() + eyeRay.Direction() * fT );

	if (!bInitialize) {
		// compute angle between new point and last point
		Wml::Vector3f v1( targetInfo.ptLastFrame - GetOrigin() );
		v1.Normalize();
		Wml::Vector3f v2( ptPlane - GetOrigin() );
		v2.Normalize();
		Wml::Vector3f vAxis( v1.Cross(v2) );
		vAxis.Normalize();
		float fAcosAngle = (v1.Dot(v2)) / vAxis.Length();
		float fAngle = Wml::Mathf::ACos( fAcosAngle );
		Wml::Matrix3f rotation;
		rotation.FromAxisAngle( vAxis, fAngle );

		RotateFrame( rotation );
		UpdateCircleTargets();
	}

	targetInfo.ptLastFrame = ptPlane;

	// return value is bunk...
	return GetOrigin();
}

void LocalRotationTool3D::UpdateCircleTargets()
{
	m_axisTargets[0] = Circle(Wml::Vector3f::ZERO, GetAxis(Wml::Frame::AxisX), m_axisTargets[0].Geometry().Radius() );
	m_axisTargets[1] = Circle(Wml::Vector3f::ZERO, GetAxis(Wml::Frame::AxisY), m_axisTargets[0].Geometry().Radius() );
	m_axisTargets[2] = Circle(Wml::Vector3f::ZERO, GetAxis(Wml::Frame::AxisZ), m_axisTargets[0].Geometry().Radius() );
}



