// Ryan Schmidt - rms@unknownroad.com
// Copyright (c) 2005-2010
// Distributed under the Boost Software License, Version 1.0.
// http://www.boost.org/LICENSE_1_0.txt


#include "StdAfx.h"
#include "ToolSet.h"
#include "Tool.h"

using namespace rmsui;


unsigned int ToolSet::m_sToolTypeGen = 0;
ToolSet::ToolType ToolSet::GenToolType()
{
	return (ToolType)++m_sToolTypeGen;
}

ToolSet::ToolType ToolSet::RegisterTool( ITool3D * pTool )
{
	ToolType nType = GenToolType();

	ToolListEntry newEntry;
	newEntry.nType = nType;
	newEntry.pTool = pTool;

	m_vTools.insert( newEntry );

	pTool->SetToolType( nType );

	return nType;
}


void ToolSet::UnRegisterTool( ToolType nType )
{
	ToolListEntry findEntry;
	findEntry.nType = nType;
	std::set<ToolListEntry>::iterator found( m_vTools.find( findEntry ) );
	if ( found == m_vTools.end () )
		return;

	found->pTool->SetToolType(0);		// invalid tool type

	m_vTools.erase( found );
	
}


ITool3D * ToolSet::FindTool( ToolType nType )
{
	ToolListEntry findEntry;
	findEntry.nType = nType;
	if ( m_vTools.find( findEntry ) == m_vTools.end () )
		return NULL;
	return m_vTools.find( findEntry )->pTool;
}


ITool3D * ToolSet::FindTool( const std::string & sToolName )
{
	std::set<ToolListEntry>::iterator cur( m_vTools.begin() ), end(m_vTools.end());
	while ( cur != end ) {
		ITool3D * pTool = cur->pTool;
		cur++;

		if ( pTool->GetToolName() == sToolName )
			return pTool;
	}

	return NULL;
}

ITool3D * ToolSet::GetNextTool( ToolType nType )
{
	ToolListEntry findEntry;
	findEntry.nType = nType;
	std::set<ToolListEntry>::iterator found( m_vTools.find( findEntry ) );
	if ( found == m_vTools.end() ) {
		if ( m_vTools.size() == 0 )
			return NULL;
		else
			return m_vTools.begin()->pTool;
	}

	found++;
	if ( found == m_vTools.end() ) 
		return m_vTools.begin()->pTool;
	else
		return found->pTool;
}

ITool3D * ToolSet::GetDefaultTool()
{
	if ( m_vTools.size() == 0 )
		return NULL;
	else
		return m_vTools.begin()->pTool;
}


