// Ryan Schmidt - rms@unknownroad.com
// Copyright (c) 2004-2010
// Distributed under the Boost Software License, Version 1.0.
// http://www.boost.org/LICENSE_1_0.txt


#ifndef RMS_OPENGL_WML_LIGHT_H
#define RMS_OPENGL_WML_LIGHT_H

#include "WmlExt.h"

#include <WmlLight.h>
#include <WmlVector3.h>

namespace Wml
{


class WMLEXT_ITEM OpenGLWmlLight : public Light
{
	// wml declaration stuff
    WmlDeclareRTTI;

public:
	OpenGLWmlLight( unsigned int nLightIndex = 0 )
		: Light(),
		  m_eType(Light::LT_POINT),
		  m_nLightIndex(nLightIndex),
		  m_ptLightPosition(Vector3f::ZERO)
	{
	}

	unsigned int & LightIndex()
	{
		return m_nLightIndex;
	}

	Vector3f & Position()
	{
		return m_ptLightPosition;
	}

	Light::Type & LightType()
	{
		return m_eType;
	}

	virtual Type GetType() const
	{
		return m_eType;
	}

	void SetupLight();

protected:
	Light::Type m_eType;
	unsigned int m_nLightIndex;
	Vector3f m_ptLightPosition;

	// these are not implemented, but are abstract in Wml::Light so we 
	//  have to have them here...

    virtual void ComputeDiffuse (const Matrix3f& rkWorldRotate,
        const Vector3f& rkWorldTranslate, float fWorldScale,
        const Vector3f* akVertex, const Vector3f* akNormal, int iQuantity,
		const bool* abVisible, ColorRGB* akDiffuse) {}

    virtual void ComputeSpecular (const Matrix3f& rkWorldRotate,
        const Vector3f& rkWorldTranslate, float fWorldScale,
        const Vector3f* akVertex, const Vector3f* akNormal, int iQuantity,
        const bool* abVisible, const Vector3f& rkCameraModelLocation,
		ColorRGB* akSpecular) {}
};



}



#endif	// RMS_OPENGL_WML_LIGHT_H