// Ryan Schmidt - rms@unknownroad.com
// Copyright (c) 2004-2010
// Distributed under the Boost Software License, Version 1.0.
// http://www.boost.org/LICENSE_1_0.txt


#include "StdAfx.h"
#include "WmlExtPlane3.h"

using namespace Wml;

template<class Real>
ExtPlane3<Real>::ExtPlane3() 
	: Plane3<Real>(), m_vOrigin(Wml::Vector3<Real>::ZERO) 
{
}

template<class Real>
ExtPlane3<Real>::ExtPlane3( const Wml::Vector3<Real> & vNormal, const Wml::Vector3<Real> & vOrigin ) 
	: Plane3<Real>( vNormal, vOrigin ), m_vOrigin(vOrigin) 
{
}

template<class Real>
ExtPlane3<Real>::ExtPlane3( const Wml::Vector3<Real> & vPoint1, 
						    const Wml::Vector3<Real> & vPoint2, 
							const Wml::Vector3<Real> & vPoint3 )
							: Plane3<Real>( vPoint1, vPoint2, vPoint3 )
{
	Normalize();
	m_vOrigin = vPoint1;
}


template<class Real>
Wml::Vector3<Real> ExtPlane3<Real>::ProjectPointToPlane( const Wml::Vector3<Real> & vPoint ) const 
{
	return vPoint - DistanceTo(vPoint) * GetNormal();
}

template<class Real>
Wml::Vector3<Real> ExtPlane3<Real>::RotatePointIntoPlane( const Wml::Vector3<Real> & vPoint ) const
{
	// find projected point in plane
	Wml::Vector3<Real> vProjected = ProjectPointToPlane( vPoint );

	// find angle between projected point and original point
	Wml::Vector3<Real> vOrig( vPoint - Origin() );
	Wml::Vector3<Real> vProj( vProjected - Origin() );

	Real fScale = vOrig.Length() / vProj.Length();
	return Origin() + vProj * fScale;

	// [RMS] following code uses rotation to do the same thing, just less efficiently!

	//Real CosTheta = vOrig.Dot(vProj) / (vOrig.Length() * vProj.Length());
	//Real Theta = std::acos( CosTheta );

	//// compute rotation axis
	//Wml::Vector3<Real> vAxis( vOrig.Cross(vProj) );
	//vAxis.Normalize();

	//// rotate point
	//Wml::Matrix3<Real> matRotate( vAxis, Theta );
	//return (matRotate * vOrig) + plane.Origin();
}


//----------------------------------------------------------------------------
// explicit instantiation
//----------------------------------------------------------------------------
namespace Wml
{
template WMLEXT_ITEM class ExtPlane3<float>;
template WMLEXT_ITEM class ExtPlane3<double>;
}
//----------------------------------------------------------------------------
