// Ryan Schmidt - rms@unknownroad.com
// Copyright (c) 2004-2010
// Distributed under the Boost Software License, Version 1.0.
// http://www.boost.org/LICENSE_1_0.txt


#ifndef __RMS_WMLEXT_POLYLINE_3_H__
#define __RMS_WMLEXT_POLYLINE_3_H__

#include "WmlExt.h"

#include <WmlVector3.h>
#include <WmlMatrix3.h>
#include <WmlAxisAlignedBox3.h>
#include "WmlExtVectorUtil.h"

#include <vector>

namespace Wml {

template<class Real>
class WMLEXT_ITEM PolyLine3
{
public:

	PolyLine3();
	PolyLine3( const PolyLine3<Real> & copy );
	~PolyLine3();

	// add/remove vertex functions
	inline unsigned int AppendVertex( const Wml::Vector3<Real> & vVertex );
	inline void ClearVertices();

	// vertex count stuff
	inline unsigned int VertexCount() const;
	inline bool IsEmpty() const;

	// element access
	inline const Wml::Vector3<Real> & Vertex( unsigned int nVertex ) const;
	inline Wml::Vector3<Real> & Vertex( unsigned int nVertex );
	inline Wml::Vector3<Real> & operator[]( int i );
	inline const Wml::Vector3<Real> & operator[] (int i ) const;
	inline const std::vector< Wml::Vector3<Real> > & Vertices() const;

	// orientation bits
	void Reverse();

	// bounding things
	void BoundingBox( Wml::AxisAlignedBox3<Real> & dest ) const;

	// translation / rotation functions
	void Transform( const Wml::Matrix3<Real> & transform );
	void Translate( Real fX, Real fY, Real fZ );

	//! simplify polyline. fToleranceCoeff is multiplied by largest bounding box dimension...
	void Simplify( float fToleranceCoeff = (Real)0.025,
				   bool bSimplifyStraightLines = true );

	// get distance to polyline
	Real SqrDistance( const Wml::Vector3<Real> & vPoint, Wml::Vector3<Real> * pNearest = NULL );

	// save and load to file
	bool Load( const char * pFilename );
	bool Save( const char * pFilename );

protected:
	std::vector< Wml::Vector3<Real> > m_vVertices;
};

typedef PolyLine3<float> PolyLine3f;
typedef PolyLine3<double> PolyLine3d;


/*
 * inlines
 */

template<class Real>
inline unsigned int PolyLine3<Real>::AppendVertex( const Wml::Vector3<Real> & vVertex )
{
	size_t nVerts = m_vVertices.size();
	m_vVertices.push_back(vVertex);
	return (unsigned int)nVerts;
}

template<class Real>
inline void PolyLine3<Real>::ClearVertices( )
{
	m_vVertices.resize(0);
}

template<class Real>
inline unsigned int PolyLine3<Real>::VertexCount() const
{
	return (unsigned int)m_vVertices.size();
}

template<class Real>
inline bool PolyLine3<Real>::IsEmpty() const
{
	return m_vVertices.empty();
}

template<class Real>
inline const Wml::Vector3<Real> & PolyLine3<Real>::Vertex( unsigned int nVertex ) const
{
	return m_vVertices[nVertex];
}

template<class Real>
inline Wml::Vector3<Real> & PolyLine3<Real>::Vertex( unsigned int nVertex )
{
	return m_vVertices[nVertex];
}

template<class Real>
inline const Wml::Vector3<Real> & PolyLine3<Real>::operator[]( int i ) const
{
	return m_vVertices[i];
}

template<class Real>
inline Wml::Vector3<Real> & PolyLine3<Real>::operator[]( int i )
{
	return m_vVertices[i];
}

template<class Real>
inline const std::vector< Wml::Vector3<Real> > & PolyLine3<Real>::Vertices() const
{
	return m_vVertices;
}




} // namespace rmssketch



#endif // __RMS_WMLEXT_POLYLINE_3_H__